C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME       - ACCES                                              +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           ACCES-ALGORITHM TO COMPUTE THE SEQUENCE (U ), U = S + a D          +
C                                                     n    n   n   n n         +
C           WHICH CONVERGES TO S FASTER THAN (S  ).                            +
C                                              n                               +
C           A GOOD ESTIMATION (D  ) OF THE ERROR IS KNOWN.                     +
C                               n                                              +
C                                                                              +
C   USAGE:                                                                     +
C           CALL ACCES (SN, DN, TN, UN, RN, INIT)                              +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           SN     - INPUT REAL VECTOR OF DIMENSION (-1:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF SN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE (S )      +
C                    (n = 0,1,...).                                    n       +
C                    THE SUBROUTINE SETS THE VALUE S  TO ZERO.                 +
C                                                   -1                         +
C                                                                              +
C           DN     - INPUT REAL VECTOR OF DIMENSION (0:M). IN THE              +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF DN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE (D ).     +
C                                                                      n       +
C                                                                              +
C           TN     - OUTPUT REAL VECTOR OF DIMENSION (-1:M). IN THE            +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF TN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS IN OUTPUT THE VALUES OF THE          +
C                    SEQUENCE (T ):  T  = 0; T  =  S + D   (n = 0,1,...).      +
C                               n     -1      n     n   n                      +
C                                                                              +
C           UN     - OUTPUT REAL VECTOR OF DIMENSION (0:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF UN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS IN OUTPUT THE VALUES OF THE          +
C                    SEQUENCE (U ) WHICH CONVERGES FASTER THAN (S ).           +
C                               n                               n              +
C                                                                              +
C           RN     - OUTPUT REAL VALUE WHICH CONTAINS, AFTER THE k-TH          +
C                    CALL, THE SIGN R    OF (S   - S).                         +
C                                    k-1      k-1                              +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE         +
C                    FIRST CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED        +
C                    TO 1 BY THE SUBROUTINE DURING THE FIRST CALL.             +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  DOUBLE PRECISION FUNCTION   DAQES                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE VECTORS TN AND UN MUST NOT BE MODIFIED     +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           SN(-1:NUMBER_OF_CALLS-1)                                           +
C           DN(0:NUMBER_OF_CALLS-1)                                            +
C           TN(-1:NUMBER_OF_CALLS-1)                                           +
C           UN(0:NUMBER_OF_CALLS-1)                                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
       SUBROUTINE ACCES (SN, DN, TN, UN, RN, INIT)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INIT
      DOUBLE PRECISION RN
      DOUBLE PRECISION SN(-1:1), DN(0:1), TN(-1:1), UN(0:1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION DAQES
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION BN
      INTEGER INIDAQ, J, N, PN
      SAVE INIDAQ, N
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         INIT   = 1
         INIDAQ = 0
         N      = -1
         SN(-1) = 0.0D0
         TN(-1) = 0.0D0
      END IF
C
C                        ... FIRST AND NEXT CALLS OF THE SUBROUTINE
C
      N     = N+1
      TN(N) = SN(N) + DN(N) 
C
C                        ... SIGN R  OF D  BY THE DAQES-ALGORITHM
C                                  n     n
C
      RN = DAQES (SN, TN, DN, INIDAQ)
      PN = -1
C
C                        ... COMPUTATION OF p(n) : 3 DIFFERENT CASES
C                            1-R * D = 0  2-R * D > 0 3-R * D < 0
C                               n   n        n   n       n   n
C
      IF (N .NE. 0 .AND. (RN*DN(N)) .NE. 0) THEN
         IF (RN * DN(N) .GT. 0) THEN
             DO 10 J = 0, N-1
                IF (RN * (TN(N) - SN(J)) .LE. 0.0D0) PN=J
   10        CONTINUE
         ELSE
             DO 20 J = 0, N-1
                IF (RN * (2.0D0*SN(N) - TN(N) - SN(J)) .LE. 0.0D0) PN=J
   20        CONTINUE
         END IF
      END IF
C
C  COMPUTATION OF b
C                  n
C
      IF (SN(N) .EQ. SN(PN)) THEN
         BN  = 1.0D0
      ELSE
         BN = (TN(N)-TN(PN)) / (SN(N)-SN(PN))
      END IF
C
C  COMPUTATION OF U
C                  n
C
      IF (BN .EQ. 1.0D0) THEN
         UN(N) = SN(N)
      ELSE
         UN(N) = SN(N) + DN(N) / (1.0D0-BN)
      END IF
      RETURN
      END
