C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - BORDER                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           COMPUTES THE SOLUTION z    OF THE BORDERED SYSTEM                  +
C                                  k+1                                         +
C           A    z    = d  IN TERMS OF THE SOLUTION z  OF THE PREVIOUS         +
C            k+1  k+1    k+1                         k                         +
C           SYSTEM A  z  = d .                                                 +
C                   k  k    k                                                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL BORDER (A, D, IR, Z, Q, EPS, INIT, IER)                       +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL SQUARE MATRIX OF DIMENSION (IR,IR).            +
C                                                                              +
C           D      - INPUT REAL VECTOR OF DIMENSION IR.                        +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           Z      - OUTPUT REAL VECTOR OF DIMENSION IR CONTAINING,            +
C                    AFTER THE k-TH CALL, IN ITS FIRST k COMPONENTS,           +
C                    THE SOLUTION z  OF A z = d .                              +
C                                  k     k k   k                               +
C                                                                              +
C           Q      - WORKING REAL VECTOR OF DIMENSION (IR*(IR+3)/2)            +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE         +
C                    FIRST CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED        +
C                    TO 1 BY THE SUBROUTINE DURING THE FIRST CALL.             +
C                    FOR A NEW APPLICATION OF THE METHOD, INIT MUST BE SET     +
C                    AGAIN TO ZERO.                                            +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO. THE BLOCK BORDERING       +
C                                  METHOD MUST BE USED, SEE THE SUBROUTINE     +
C                                  BLBORD.                                     +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO NUMBER_OF_CALLS.      +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       - SUBROUTINE                QBORD                                      +
C       - DOUBLE PRECISION FUNCTION INNERC                                     +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION. THE VECTORS Z AND Q MUST NOT BE MODIFIED BY    +
C          THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.           +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=NUMBER_OF_CALLS) ARE:               +
C           A(IR,IR)                                                           +
C           D(IR)                                                              +
C           Z(IR)                                                              +
C           Q(IR*(IR+3)/2)                                                     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE BORDER (A, D, IR, Z, Q, EPS, INIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IR
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1), Q(1), D(1), Z(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION INNERC
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION COEFF
      INTEGER INDM, INIQBO, J
      SAVE INDM, INIQBO
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0
         INIT   = 1
         INIQBO = 0
         INDM   = 0
         IF (DABS(A(1,1)) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         Z(1)   = D(1) / A(1,1)
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE NUMBER_OF_CALLS - 1
C
      INDM = INDM + 1
      IF (INDM .GT. IR) THEN
         IER = 300
         RETURN
      END IF
C
C                        ... COMPUTES q 
C                                      k
C
      CALL QBORD (A, IR, Q, EPS, INIQBO, IER)
      IF (IER .EQ. 100) RETURN
C
C                        ... COMPUTES THE VALUE OF THE SOLUTION z
C                                                                k+1
C
      COEFF   = (D(INDM+1) - INNERC (A, INDM+1, Z, 1, IR, INDM, 0))/ 
     *          Q(INDM*(INDM+3)/2)
      Z(INDM+1) = COEFF 
      DO 10 J = 1, INDM
         Z(J) = Z(J) + COEFF * Q(J+(INDM-1)*(INDM+2)/2) 
   10 CONTINUE
      RETURN
      END      
