C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   FUNCTION NAME       - DAQES                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           DOUBLE PRECISION FUNCTION WHICH DETERMINES ASYMPTOTICALLY          +
C           THE SIGN R  OF S  - S.                                             +
C                     n     n                                                  +
C           THE FUNCTION USES THE DAQES-ALGORITHM.                             +
C           THE CONSTANT c TO COMPUTE T (j) = c * T  + (1 - c) * S             +
C                                      c           j              j            +
C           (j = k, ..., n)                                                    +
C           IS SET TO -1/a   (FOR THE ASSERTION A(k,n))                        +
C                         k                                                    +
C           AND    TO  1/a   (FOR THE ASSERTION B(k,n))                        +
C                         k                                                    +
C           WHERE (a ) IS THE AUXILIARY STRICTLY DECREASING SEQUENCE           +
C                   k                                                          +
C                                             -k                               +
C           CONVERGING TO ZERO GIVEN BY a  = 2  .                              +
C                                        k                                     +
C                                                                              +
C   USAGE:                                                                     +
C           DAQES (SN, TN, DN, INIT)                                           +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           SN     - INPUT REAL VECTOR OF DIMENSION (-1:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF SN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE S .       +
C                                                                     n        +
C                                                                              +
C           TN     - INPUT REAL VECTOR OF DIMENSION (-1:M). IN THE             +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF TN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE T .       +
C                                                                     n        +
C                                                                              +
C           DN     - INPUT REAL VECTOR OF DIMENSION (0:M). IN THE              +
C                    CALLING PROCEDURE THE UPPER-BOUND DIMENSION OF DN         +
C                    MUST BE GREATER OR EQUAL TO (NUMBER_OF_CALLS-1).          +
C                    THIS VECTOR CONTAINS THE VALUES OF THE SEQUENCE D .       +
C                                                                     n        +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE         +
C                    FIRST CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED        +
C                    TO 1 BY THE SUBROUTINE DURING THE FIRST CALL.             +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  LOGICAL FUNCTION   IASSER                                           +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION.                                             +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           SN(-1:NUMBER_OF_CALLS-1)                                           +
C           TN(-1:NUMBER_OF_CALLS-1)                                           +
C           DN(0:NUMBER_OF_CALLS-1)                                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      DOUBLE PRECISION FUNCTION DAQES (SN, TN, DN, INIT) 
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INIT
      DOUBLE PRECISION SN(-1:1), TN(-1:1), DN(0:1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      LOGICAL IASSER
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      LOGICAL ASSA, ASSB
      DOUBLE PRECISION C, RNSIGN
      INTEGER K, N
      SAVE K, N, RNSIGN
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         INIT   = 1
         K      = 0
         N      = -1
         RNSIGN = 1.0D0
      END IF
C
C                        ... FIRST AND NEXT CALLS OF THE SUBROUTINE
C
      N    = N+1
C
C                        ... EVALUATE THE ASSERTION A(k,n)
C
      C    = -2.0D0**K
      ASSA = IASSER (SN, TN, C, K, N)
      IF (ASSA) THEN
C
C                        ... A(k,n) IS TRUE
C
         DAQES  = DSIGN(RNSIGN, DN(N))
         RNSIGN = DAQES
         RETURN
      END IF
C
C                        ... EVALUATE THE ASSERTION B(k,n)
C
      C    = 2.D0**K
      ASSB = IASSER (SN, TN, C, K, N)
      IF (ASSB) THEN
C
C                        ... B(k,n) IS TRUE
C
         DAQES  = -DSIGN(RNSIGN, DN(N))
         RNSIGN = DAQES
         RETURN
      END IF
C
C                        ... A(k,n) AND B(k,n) ARE FALSE
C
      DAQES = RNSIGN
      K = N + 1
      RETURN
      END
