C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - EPSRHA                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SIMULTANEOUS SCALAR SEQUENCE EXTRAPOLATION.                        +
C           THE SUBROUTINE PERFORMS THE ESPILON-ALGORITHM, THE FIRST AND THE   +
C           SECOND GENERALIZATIONS OF THE EPSILON-ALGORITHM AND THE RHO        +
C           ALGORITHM.                                                         +
C           THE EPSILON-ALGORITHM IS USED FOR TRANSFORMING THE SEQUENCE GIVEN  +
C           AS THE COMPONENT 0 OF EPSINI. THE FIRST GENERALIZATION OF THE      +
C           EPSILON-ALGORITHM IS USED FOR TRANSFORMING THE SEQUENCE GIVEN AS   +
C           COMPONENT 1 OF EPSINI WITH THE COMPONENT 1 OF XINIT AS AN          +
C           AUXILIARY SEQUENCE. THE SECOND GENERALIZATION OF THE EPSILON       +
C           ALGORITHM IS USED FOR TRANSFORMING THE SEQUENCE GIVEN AS THE       +
C           COMPONENT 2 OF EPSINI WITH THE COMPONENT 2 OF XINIT AS AN          +
C           AUXILIARY SEQUENCE. THE RHO-ALGORITHM IS USED FOR TRANSFORMING     +
C           THE SEQUENCE GIVEN AS THE COMPONENT 3 OF EPSINI WITH THE COMPONENT +
C           3 OF XINIT.                                                        +
C           THE SUBROUTINE CHOOSES BETWEEN THE NORMAL AND THE PARTICULAR       +
C           RULES OF WYNN ACCORDING TO THE DESIRED PERMISSIBLE DEGREE OF       +
C           CANCELLATION.                                                      +
C                                                                              +
C   USAGE:                                                                     +
C           CALL EPSRHA (EPSVEC, MAXCOL, INIT, EPS, EPSINI, NDIGIT,            +
C              MAXSIN, NSING, INDSAV, ABVEC, XAUX, XINIT, IER)                 +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           EPSVEC - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF EPSVEC MUST BE (0:3,0:IE), WITH IE GREATER  +
C                    OR EQUAL TO MAXCOL (IF MAXCOL IS EVEN) OR TO (MAXCOL-1)   +
C                    (IF MAXCOL IS ODD). THIS MATRIX CONTAINS, IN EACH ROW,    +
C                    AFTER THE k-TH CALL, THE LAST COMPUTED BACKWARD DIAGONAL  +
C                    OF THE ARRAY FOR THE CORRESPONDING ALGORITHM:             +
C                                                                              +
C                                          (k-1)   (k-2)        (0)            +
C                     - IF k <= MAXCOL+1 eps   , eps    ,..., eps              +
C                                           0       1            k-1           +
C                                                                              +
C                                          (k-1)   (k-2)        (k-1-MAXCOL)   +
C                     - IF k > MAXCOL+1  eps   , eps    ,..., eps              +
C                                           0       1            MAXCOL        +
C                    THE ROW 0 CONTAINS THE RESULTS FOR THE EPSILON-ALGORITHM, +
C                    THE ROW 1 CONTAINS THE RESULTS FOR THE FIRST GENERALIZ.   +
C                    OF THE EPSILON-ALGORITHM, THE ROW 2 CONTAINS THE RESULTS  +
C                    FOR THE SECOND GENERALIZATION OF THE EPSILON-ALGORITHM    +
C                    AND THE ROW 3 CONTAINS THE RESULTS FOR THE RHO-ALGORITHM. +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE ARRAY THAT THE USER WANTS TO COMPUTE.              +
C                                           (n)                                +
C                    (THE LOWER INDEX OF eps   ). IF MAXCOL IS ODD THE         +
C                                            m                                 +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EPSINI - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUES      +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    THE FIRST VALUE IS USED IN THE EPSILON ALGORITHM,         +
C                    THE SECOND VALUE IN THE FIRST GENERALIZATION OF THE       +
C                    EPSILON ALGORITHM,                                        +
C                    THE THIRD VALUE IN THE SECOND GENERALIZATION OF THE       +
C                    EPSILON ALGORITHM,                                        +
C                    THE FOURTH VALUE IN THE RHO ALGORITHM.                    +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE, IN     +
C                    EACH COMPONENT THE CORRESPONDING                          +
C                                                                              +
C                       (0)                                                    +
C                    eps            IF  k <= MAXCOL+1, k ODD                   +
C                       k-1                                                    +
C                                                                              +
C                       (1)                                                    +
C                    eps            IF  k <  MAXCOL+1, k EVEN                  +
C                       k-2                                                    +
C                                                                              +
C                       (k-1-MAXCOL)                                           +
C                    eps            IF  k >  MAXCOL+1                          +
C                       MAXCOL                                                 +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                      (0)   (1)   (0)   (1)       (0)      (1)      (2)       +
C                    eps , eps , eps , eps ,..., eps    , eps    , eps   , ... +
C                       0     0     2     2         MAXCOL   MAXCOL   MAXCOL   +
C                                                                              +
C           NDIGIT - INPUT INTEGER VALUE. IT REPRESENTS THE NUMBER OF DIGITS   +
C                    THAT THE USER ALLOWS TO LOSE DUE TO THE CANCELLATION      +
C                    ERROR. IF FOR ANY INDEX k AND FOR ANY n THE RELATION      +
C                         (n+1)   (n)    (n)     -NDIGIT                       +
C                    | (eps   - eps  )/eps  | < 10      HOLDS, THEN THE        +
C                          k       k      k                                    +
C                    PARTICULAR RULES OF WYNN SHALL BE USED TO COMPUTE THE     +
C                               (n-1)                                          +
C                    VALUE OF eps    AT THE NEXT CALL. BUT IF, IN THE NEXT     +
C                                k+3         (n+2)   (n+1)  (n+1)     -NDIGIT  +
C                    CALL, WE ALSO HAVE | (eps   - eps  )/eps    | < 10        +
C                                            k       k      k                  +
C                    THEN THE SUBROUTINE STOPS DUE TO A NON ISOLATED           +
C                    SINGULARITY.                                              +
C                                                                              +
C           MAXSIN - INPUT INTEGER GIVING THE MAXIMUM NUMBER OF SINGULARITIES  +
C                    THAT THE SUBROUTINE CAN STORE.                            +
C                                                                              +
C           NSING  - OUTPUT INTEGER VECTOR. EACH COMPONENT REPRESENTS (FOR THE +
C                    CORRESPONING ALGORITHM) THE TOTAL NUMBER OF SINGULARITIES +
C                    DETECTED IN ALL THE PRECEDING CALLS OF THE SUBROUTINE.    +
C                                                                              +
C           INDSAV - WORKING INTEGER MATRIX IN WHICH THE SUBROUTINE SAVES THE  +
C                    LOWER INDEXES OF THE SINGULARITIES. IN THE CALLING        +
C                    PROCEDURE ITS SECOND DIMENSION MUST BE GREATER OR EQUAL   +
C                    TO MAXSIN.                                                +
C                                                                              +
C           ABVEC  - WORKING REAL MATRIX. IN THE FIRST MAXSIN ELEMENTS OF EACH +
C                    ROW, THE SUBROUTINE SAVES THE VALUES OF A AND IN THE      +
C                    SUBSEQUENT MAXSIN ELEMENTS THE VALUES OF B. IN THE        +
C                    CALLING PROCEDURE THE SECOND DIMENSION OF ABVEC MUST BE   +
C                    GREATER OR EQUAL TO 2*MAXSIN.                             +
C                                                                              +
C           XAUX   - WORKING REAL MATRIX TO STORE BY ROWS THE VALUES OF XINIT  +
C                    GIVEN AT EACH CALL BY THE USER. IN THE CALLING PROCEDURE  +
C                    THE SECOND DIMENSION OF XAUX MUST BE THE SAME THAN THAT   +
C                    OF THE COLUMNS OF EPSVEC.                                 +
C                                                                              +
C           XINIT  - INPUT REAL VECTOR. IT MUST CONTAIN THE ELEMENT x    OF    +
C                                                                    k-1       +
C                    THE AUXILIARY SEQUENCES, BEFORE THE  k-TH CALL OF         +
C                    THE SUBROUTINE.                                           +
C                    THE COMPONENT 1 OF THIS VECTOR IS FOR THE AUXILIARY       +
C                    SEQUENCE CORRESPONDING TO THE FIRST GENERALIZATION OF     +
C                    THE EPSILON-ALGORITHM, THE COMPONENT 2 IS FOR THE         +
C                    AUXILIARY SEQUENCE CORRESPONDING TO THE SECOND GENERALIZ. +
C                    OF THE EPSILON-ALGORITHM AND THE COMPONENT 3 IS FOR THE   +
C                    RHO-ALGORITHM.                                            +
C                                                                              +
C           IER    - OUTPUT INTEGER VECTOR REPORTING THE WARNING/ERROR CODE    +
C                    FOR THE CORRESPONDING ALGORITHM.                          +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 1100    TOO MANY SINGULARITIES TO BE STORED.        +
C                                  INCREASE THE VALUE OF MAXSIN AND THE        +
C                                  DIMENSION OF THE VECTORS ABVEC AND INDSAV.  +
C                    IER = 1200    SINGULARITY NON ISOLATED. IMPOSSIBLE TO     +
C                                  CONTINUE.                                   +
C                    IN EACH CALL THE SUBROUTINE PERFORMS ONLY THE ALGORITHMS  +
C                    WHOSE CORRESPONDING IER COMPONENT IS ZERO.                +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE MATRICES EPSVEC, INDSAV, ABVEC AND XAUX    +
C          MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF   +
C          THE SUBROUTINE.                                                     +
C                                    (0)                                       +
C       -  TO OBTAIN THE VALUE OF eps   (2*J <= MAXCOL) THE USER MUST CALL     +
C                                    2*J                                       +
C          THIS SUBROUTINE (2*J+1) TIMES.                                      +
C       -  IT IS UNUSEFULL TO ASSIGN MAXSIN A VALUE GREATER THAT (MAXCOL-3).   +
C          WHEN MAXCOL<=2, THE PARTICULAR RULES CANNOT BE USED.                +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS ARE:                                    +
C           EPSVEC(0:3,0:MAXCOL)                                               +
C           NSING(0:3)                                                         +
C           INDSAV(0:3,MAXSIN)                                                 +
C           EPSINI(0:3)                                                        +
C           ABVEC(0:3,2*MAXSIN)                                                +
C           XAUX(3,0:MAXCOL)                                                   +
C           XINIT(3)                                                           +
C           IER(0:3)                                                           +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE EPSRHA (EPSVEC, MAXCOL, INIT, EPS, EPSINI, NDIGIT,
     *       MAXSIN, NSING, INDSAV, ABVEC, XAUX, XINIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INIT, MAXCOL, MAXSIN, NDIGIT
      INTEGER IER(0:3), INDSAV(0:3,1), NSING(0:3)
      DOUBLE PRECISION EPS
      DOUBLE PRECISION EPSINI(0:3), EPSVEC(0:3,0:1), XAUX(3,0:1), 
     *       XINIT(3), ABVEC(0:3,1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER I, IND, INDM, INDS, INDXL, INDXU, MAXCP1
      INTEGER LISTBEG(0:3), LISTEND(0:3)
      DOUBLE PRECISION ANDIG, RD
      DOUBLE PRECISION AUX0(0:3), AUX1(0:3), AUX2(0:3), COEFF(0:3),
     *     DTMP(0:3)
      LOGICAL*1 IAC(0:3), IBC(0:3), IEC(0:3), LOCINS(0:3), NEWINS(0:3)
      SAVE LISTBEG, LISTEND, INDM, MAXCP1,
     *     LOCINS, NEWINS, IEC, IAC, IBC, 
     *     ANDIG
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         DO 10 I = 0, 3
            IER(I)      = 0 
            NSING(I)    = 0
            LOCINS(I)   = .FALSE.
            NEWINS(I)   = .FALSE.
            IEC(I)      = .FALSE.
            IAC(I)      = .FALSE.
            IBC(I)      = .FALSE.
            EPSVEC(I,0) = EPSINI(I)
            IF (I .NE. 0) XAUX(I,0)   = XINIT(I)
   10    CONTINUE
         INIT   = 1
         ANDIG  = 10.0D0 ** (-FLOAT(NDIGIT))
         INDM   = 0
C
C                        ... IF MAXCOL IS ODD IT IS CHANGED TO
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
         MAXCP1 = MAXCOL + 1
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 I = 0, 3
            EPSVEC(I,0) = EPSINI(I)
            IF (I .NE. 0) XAUX(I,0)   = XINIT(I)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE BACKWARD DIAGONAL TO 
C                            BE COMPUTED AND THE LOWER INDEX m OF THE 
C                                                          (0)
C                            LAST VALUE OF THIS DIAGONAL eps    TO BE 
C                                                           m
C                            COMPUTED IN THE CALL.
C
      INDM = INDM + 1 
C
C                        ... IF INDM .GT. MAXCOL THEN THE ELEMENT 
C                               (NUMBER_OF_CALLS-MAXCOL-1)
C                            eps                          IS COMPUTED.
C                               MAXCOL
C
      IND = MIN0(INDM,MAXCOL)
C
C                        ... STORE THE XINIT VALUE FOR THE RHO AND 
C                            THE GENERALIZATIONS OF THE EPSILON-ALGORITHM.
C
      INDXL = MOD(INDM,MAXCP1)
      DO 30 I = 1, 3
         XAUX(I,INDXL) = XINIT(I)
   30 CONTINUE
C
      DO 40 I = 0, 3
         IF (IER(I).EQ. 0) THEN
C
C                        ... IF THERE IS NO LOCAL INSTABILITY PRESENT,
C                            RESET THE INDEXES LISTBEG AND LISTEND OF
C                            THE VECTOR INDSAV.
C
            IF (.NOT. LOCINS(I)) THEN
               LISTBEG(I) = 1
               LISTEND(I) = 0
            END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
            AUX1(I)   = EPSINI(I)
         ELSE
            IER(I) = 200
         END IF
   40 CONTINUE
C
      IF (IER(0).NE.0 .AND. IER(1).NE.0 .AND. IER(2).NE.0 
     *        .AND. IER(3).NE.0) RETURN
C  
C                        ... SET THE VALUES OF COEFF NOT DEPENDING ON 
C                            THE VALUE OF INDS.
C
      COEFF(0) = 1.0D0
      INDXU = MOD(INDM-1,MAXCP1)
      COEFF(2) = XAUX(2,INDXL)-XAUX(2,INDXU) 
C
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THE ASCENDING DIAGONAL.
C
      DO 60 INDS = 0, IND - 1
C  
C                        ... SET THE VALUES OF COEFF DEPENDING ON THE
C                            VALUE OF INDS.
C
         INDXL = MOD(INDM,MAXCP1)
         INDXU = MOD(INDM-INDS-1,MAXCP1)
         COEFF(3) = XAUX(3,INDXL)-XAUX(3,INDXU) 
         INDXL = MOD(INDM-INDS,MAXCP1)
         COEFF(1) = XAUX(1,INDXL)-XAUX(1,INDXU) 
C
C                        ... STARTING LOOP FOR THE FOUR ALGORITHMS.
C
         DO 50 I = 0, 3
            IF (IER(I).EQ. 0) THEN
C  
C                        ... COMPUTE THE DENOMINATOR.
C
               AUX0(I) = AUX1(I) - EPSVEC(I,INDS)
C  
C                        ... CONTROL FOR A NEW POINT OF INSTABILITY.
C
               RD = DMAX1(DABS(AUX1(I)),DABS(EPSVEC(I,INDS)))
               IF (DABS(EPSVEC(I,INDS)) .GT. EPS .AND. .NOT.IEC(I) 
     *            .AND. (RD .LT. EPS .OR. DABS(AUX0(I)/RD) .LT. ANDIG) 
     *            .AND. INDS .LT. IND-1 .AND. INDS .LT. MAXCOL-2) THEN
                  NEWINS(I)  = .TRUE.
                  NSING(I)   = NSING(I)+1
                  LISTEND(I) = LISTEND(I)+1
                  IF (LISTEND(I) .GT. MAXSIN) IER(I) = 1100
               END IF 
C
               IF ((LOCINS(I).OR.NEWINS(I)) .AND. IER(I).EQ.0) THEN
C
C                        ... WHEN THERE IS A LOCAL INSTABILITY PRESENT
C                            OR A NEW POINT OF INSTABILITY WE USE
C                            THE SINGULAR RULE OF THE ALGORITHM.
                  IF (NEWINS(I)) THEN
C   
C                        ... THERE IS A NEW POINT OF INSTABILITY ...
C
                     IF (LOCINS(I)) THEN
C  
C                        ... AND THERE IS A LOCAL INSTABILITY PRESENT.
C
C  
C                        ... IF INDS COINCIDES WITH THE FIRST ELEMENT
C                            STORED IN INDSAV THEN THERE IS A NON ISOLATED 
C                            SINGULARITY AND IT IS IMPOSSIBLE TO CONTINUE.
C
                        IF (INDS .EQ. INDSAV(I,LISTBEG(I))) IER(I)=1200
                     END IF
                     IF (IER(I).EQ. 0) THEN
C  
C                        ... LOCAL INSTABILITY PRESENT IS TRUE.
C
                        LOCINS(I) = .TRUE.
C
C                        ... WE MUST COMPUTE A!
C
                        IAC(I)    = .TRUE.
C  
C                        ... NEW POINT OF INSTABILITY TREATED!
C
                        NEWINS(I) = .FALSE.
C
C                        ... ADD THE CURRENT INDS TO THE END OF INDSAV.
C
                        INDSAV(I,LISTEND(I)) = INDS
                     END IF
                  END IF
C   
C                        ... WHEN THE CONTROL ON A NEW POINT OF INSTABILITY
C                            IS MADE, THE CONTROL FOR THE COMPUTATION OF E
C                            IS PERFORMED.
C
                  IF (IEC(I) .AND. IER(I) .EQ. 0) THEN
C
C                        ... WE MUST COMPUTE E AND STORE IT IN AUX0.
C
                     DTMP(I)  = DTMP(I) + ABVEC(I,LISTBEG(I)+MAXSIN) - 
     *                    ABVEC(I,LISTBEG(I))
                     IF (DABS(EPSVEC(I,INDS-1)) .LT. EPS) IER(I) = 100
                     IF (IER(I).EQ. 0) THEN            
                         AUX0(I) = DTMP(I) / (COEFF(I) + 
     *                         DTMP(I) / EPSVEC(I,INDS-1))
C  
C                        ... E COMPUTED!
C
                         IEC(I)     = .FALSE.
C
C                        ... INCREASE THE POINTER ON THE VECTOR INDSAV.
C
                         LISTBEG(I) = LISTBEG(I) + 1
C
C                        ... IF, IN THE VECTOR INDSAV, ALL THE STORED
C                            INDEXES HAVE BEEN CONSIDERED, THERE IS NO
C                            LOCAL INSTABILITY PRESENT.
C
                         IF (LISTBEG(I) .GT. LISTEND(I)) 
     *                         LOCINS(I) = .FALSE.
                     END IF
C
                  ELSE
C
C                        ... IF WE DON'T COMPUTE E, THE NORMAL RULE IS
C                            CONSIDERED.
C
                     IF (DABS(AUX0(I)) .LT. EPS) IER(I) = 100
                     IF (IER(I).EQ. 0) THEN
                        AUX0(I) = COEFF(I) / AUX0(I) 
                        IF (INDS.NE.0) AUX0(I)=AUX0(I)+EPSVEC(I,INDS-1)
                     END IF
                  END IF
C
                  IF (LOCINS(I) .AND. IER(I) .EQ. 0) THEN
C   
C                        ... THERE IS A LOCAL INSTABILITY PRESENT ...
C
                     IF (IAC(I)) THEN
C
C                        ... AND WE MUST COMPUTE AND STORE THE VALUE 
C                            A = COEFF * W / (1 - W/C).
C
                        IF (INDS.NE.0) THEN
                           IF (DABS(AUX0(I)) .LT. EPS) IER(I) = 100
                           IF (IER(I).EQ. 0) 
     *                     ABVEC(I,LISTEND(I))=COEFF(I)*EPSVEC(I,INDS-1) 
     *                          / (1.0D0 - EPSVEC(I,INDS-1) / AUX0(I))
                        ELSE
                           ABVEC(I,LISTEND(I)) = 0.0D0
                        END IF
                        IF (IER(I).EQ. 0) THEN
C  
C                        ... A COMPUTED!
C
                           IAC(I) = .FALSE.
C  
C                        ... WE MUST COMPUTE ALSO B!
C
                           IBC(I) = .TRUE.
                        END IF
C
C                        ... IF WE DON'T COMPUTE A ...
C
                     ELSE IF (IBC(I)) THEN
C
C                        ... BUT WE MUST COMPUTE AND STORE THE VALUE
C                            B = COEFF * N / (1 - N/C).
C
                        IF (DABS(AUX1(I)) .LT. EPS) IER(I) = 100
                        IF (IER(I).EQ. 0) 
     *                  ABVEC(I,LISTEND(I)+MAXSIN) = COEFF(I) * 
     *                    EPSVEC(I,INDS)/(1.0D0-EPSVEC(I,INDS)/AUX1(I))
C  
C                        ... B COMPUTED!
C
                        IF (IER(I).EQ. 0) IBC(I) = .FALSE.
C
C                        ... IF WE DON'T COMPUTE A AND WE DON'T COMPUTE B ...
C
                     ELSE IF (INDS .NE. 0 .AND. INDS .EQ.
     *                          (INDSAV(I,LISTBEG(I))+1)) THEN
C
C                        ... AND THE CURRENT INDEX INDS IS THE NEXT
C                            INDEX OF INSTABILITY IN INDSAV WE COMPUTE 
C                            D = COEFF * S / (1 - S/C) ...
C
                        IF (DABS(EPSVEC(I,INDS)) .LT. EPS) IER(I) = 100
                        IF (IER(I).EQ. 0) 
     *                  DTMP(I) = COEFF(I) * AUX1(I) / 
     *                    (1.0D0 - AUX1(I) / EPSVEC(I,INDS))
C  
C                        ... AND WE MUST COMPUTE E!
C
                        IF (IER(I).EQ. 0) IEC(I) = .TRUE.
                     END IF
                  END IF 
               ELSE
C
C                        ... WHEN THERE IS NO LOCAL INSTABILITY PRESENT
C                            NOR A NEW POINT OF INSTABILITY WE USE
C                            THE NORMAL RULE OF THE ALGORITHM.
C  
                  IF (DABS(AUX0(I)) .LT. EPS) IER(I) = 100
                  IF (IER(I) .EQ. 0) THEN
                     AUX0(I) = COEFF(I) / AUX0(I) 
                     IF (INDS .NE. 0) AUX0(I) = AUX0(I)+EPSVEC(I,INDS-1)
                  END IF
               END IF
C  
C                        ... SHIFT THE RHOMBUS.
C
               IF (IER(I) .EQ. 0) THEN
                  IF (INDS .NE. 0) EPSVEC(I,INDS-1) = AUX2(I)
                  AUX2(I) = AUX1(I)
                  AUX1(I) = AUX0(I)
               END IF
C  
C                        ... END LOOP FOR THE FOUR ALGORITHMS.
C
            END IF
   50    CONTINUE
C  
C                        ... END LOOP FOR INDS.
C
   60 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES OF THE DIAGONAL.
C
      DO 70 I = 0, 3
         IF (IER(I) .EQ. 0) THEN
            EPSVEC(I,IND-1) = AUX2(I)
            EPSVEC(I,IND)   = AUX1(I)
C  
C                        ... SET THE RESULT VALUE.
C
            IF (MOD(IND,2) .EQ. 0) THEN
               EPSINI(I) = EPSVEC(I,IND)
            ELSE
               EPSINI(I) = EPSVEC(I,IND-1)
            END IF
         END IF
   70 CONTINUE
      RETURN
      END
