C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - EPSRHO                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION.                                     +
C           THE USER CAN CHOOSE BETWEEN THE ESPILON-ALGORITHM, THE FIRST       +
C           OR THE SECOND GENERALIZATION OF THE EPSILON-ALGORITHM AND THE      +
C           RHO ALGORITHM. THE SUBROUTINE PERFORMS THE NORMAL OR THE           +
C           PARTICULAR RULES OF WYNN ACCORDING TO THE DESIRED PERMISSIBLE      +
C           DEGREE OF CANCELLATION.                                            +
C                                                                              +
C   USAGE:                                                                     +
C           CALL EPSRHO (EPSVEC, MAXCOL, INIT, EPS, EPSINI, NDIGIT,            +
C              MAXSIN, NSING, INDSAV, ABVEC, XAUX, XINIT, IALG, IER)           +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           EPSVEC - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF EPSVEC MUST BE (0:IE), WITH IE GREATER       +
C                    OR EQUAL TO MAXCOL (IF MAXCOL IS EVEN) OR TO (MAXCOL-1)   +
C                    (IF MAXCOL IS ODD). THIS VECTOR CONTAINS, AFTER THE       +
C                    k-TH CALL THE LAST COMPUTED BACKWARD DIAGONAL OF THE      +
C                    ARRAY:                                                    +
C                                                                              +
C                                          (k-1)   (k-2)        (0)            +
C                     - IF k <= MAXCOL+1 eps   , eps    ,..., eps              +
C                                           0       1            k-1           +
C                                                                              +
C                                          (k-1)   (k-2)        (k-1-MAXCOL)   +
C                     - IF k > MAXCOL+1  eps   , eps    ,..., eps              +
C                                           0       1            MAXCOL        +
C                            (n)                                               +
C                    WHERE eps  REPRESENTS EITHER THE EPSILON ALGORITHM, ITS   +
C                             m                                                +
C                    FIRST TWO GENERALIZATIONS OR THE RHO-ALGORITHM, ACCORDING +
C                    TO THE USER'S CHOICE.                                     +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE ARRAY THAT THE USER WANTS TO COMPUTE.              +
C                                           (n)                                +
C                    (THE LOWER INDEX OF eps   ). IF MAXCOL IS ODD THE         +
C                                            m                                 +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EPSINI - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                       (0)                                                    +
C                    eps            IF  k <= MAXCOL+1, k ODD                   +
C                       k-1                                                    +
C                                                                              +
C                       (1)                                                    +
C                    eps            IF  k <  MAXCOL+1, k EVEN                  +
C                       k-2                                                    +
C                                                                              +
C                       (k-1-MAXCOL)                                           +
C                    eps            IF  k >  MAXCOL+1                          +
C                       MAXCOL                                                 +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                      (0)   (1)   (0)   (1)       (0)      (1)      (2)       +
C                    eps , eps , eps , eps ,..., eps    , eps    , eps   , ... +
C                       0     0     2     2         MAXCOL   MAXCOL   MAXCOL   +
C                                                                              +
C           NDIGIT - INPUT INTEGER VALUE. IT REPRESENTS THE NUMBER OF DIGITS   +
C                    THAT THE USER ALLOWS TO LOSE DUE TO THE CANCELLATION      +
C                    ERROR. IF FOR ANY INDEX k AND FOR ANY n THE RELATION      +
C                         (n+1)   (n)    (n)     -NDIGIT                       +
C                    | (eps   - eps  )/eps  | < 10      HOLDS, THEN THE        +
C                          k       k      k                                    +
C                    PARTICULAR RULES OF WYNN SHALL BE USED TO COMPUTE THE     +
C                               (n-1)                                          +
C                    VALUE OF eps    AT THE NEXT CALL. BUT IF, IN THE NEXT     +
C                                k+3         (n+2)   (n+1)  (n+1)     -NDIGIT  +
C                    CALL, WE ALSO HAVE | (eps   - eps  )/eps    | < 10        +
C                                            k       k      k                  +
C                    THEN THE SUBROUTINE STOPS DUE TO A NON ISOLATED           +
C                    SINGULARITY.                                              +
C                                                                              +
C           MAXSIN - INPUT INTEGER GIVING THE MAXIMUM NUMBER OF SINGULARITIES  +
C                    THAT THE SUBROUTINE CAN STORE.                            +
C                                                                              +
C           NSING  - OUTPUT INTEGER VALUE. IT REPRESENTS THE TOTAL NUMBER OF   +
C                    SINGULARITIES DETECTED IN ALL THE PRECEDING CALLS OF THE  +
C                    SUBROUTINE.                                               +
C                                                                              +
C           INDSAV - WORKING INTEGER VECTOR IN WHICH THE SUBROUTINE SAVES      +
C                    THE LOWER INDEXES OF THE SINGULARITIES. IN THE CALLING    +
C                    PROCEDURE ITS DIMENSION MUST BE GREATER OR EQUAL TO       +
C                    MAXSIN.                                                   +
C                                                                              +
C           ABVEC  - WORKING REAL VECTOR. IN THE FIRST MAXSIN ELEMENTS THE     +
C                    SUBROUTINE SAVES THE VALUES OF A AND IN THE SUBSEQUENT    +
C                    MAXSIN ELEMENTS THE VALUES OF B. IN THE CALLING PROCEDURE +
C                    THE DIMENSION OF ABVEC MUST BE GREATER OR EQUAL TO        +
C                    2*MAXSIN.                                                 +
C                                                                              +
C           XAUX   - WORKING REAL VECTOR TO STORE THE VALUES OF XINIT GIVEN    +
C                    AT EACH CALL BY THE USER. IN THE CALLING PROCEDURE THE    +
C                    DIMENSION OF XAUX MUST BE THE SAME THAN THAT OF EPSVEC,   +
C                    IF THE USER USES ONE OF THE GENERALIZATIONS OF THE        +
C                    EPSILON-ALGORITHM OR THE RHO-ALGORITHM, OTHERWISE IT MUST +
C                    BE (0:1).                                                 +
C                                                                              +
C           XINIT  - INPUT REAL VALUE. IT MUST CONTAIN THE ELEMENT x    OF     +
C                                                                   k-1        +
C                    THE AUXILIARY SEQUENCE, BEFORE THE  k-TH CALL OF          +
C                    THE SUBROUTINE. VALID ONLY FOR THE RHO-ALGORITHM OR       +
C                    THE GENERALIZATIONS OF THE EPSILON-ALGORITHM. FOR THE     +
C                    NORMAL EPSILON-ALGORITHM ITS VALUE IS NOT USED.           +
C                                                                              +
C           IALG   - INPUT/OUTPUT INTEGER FLAG.                                +
C                    IF IALG .EQ. 0 THE SUBROUTINE PERFORMS THE EPSILON-ALG.   +
C                    IF IALG .EQ. 1 THE SUBROUTINE PERFORMS THE FIRST GEN.     +
C                    IF IALG .EQ. 2 THE SUBROUTINE PERFORMS THE SECOND GEN.    +
C                    IF IALG .EQ. 3 THE SUBROUTINE PERFORMS THE RHO-ALG.       +
C                    IF IALG .LT. 0 .OR. IALG .GT. 3 THE SUBROUTINE PUTS       +
C                    IALG TO ZERO AND THUS IT PERFORMS THE EPSILON-ALGORITHM.  +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 1100    TOO MANY SINGULARITIES TO BE STORED.        +
C                                  INCREASE THE VALUE OF MAXSIN AND THE        +
C                                  DIMENSION OF THE VECTORS ABVEC AND INDSAV.  +
C                    IER = 1200    SINGULARITY NON ISOLATED. IMPOSSIBLE TO     +
C                                  CONTINUE.                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE VECTORS EPSVEC, INDSAV, ABVEC AND XAUX     +
C          MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF   +
C          THE SUBROUTINE.                                                     +
C                                    (0)                                       +
C       -  TO OBTAIN THE VALUE OF eps   (2*J <= MAXCOL) THE USER MUST CALL     +
C                                    2*J                                       +
C          THIS SUBROUTINE (2*J+1) TIMES.                                      +
C       -  IT IS UNUSEFULL TO ASSIGN MAXSIN A VALUE GREATER THAT (MAXCOL-3).   +
C          WHEN MAXCOL<=2, THE PARTICULAR RULES CANNOT BE USED.                +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS ARE:                                    +
C           EPSVEC(0:MAXCOL)                                                   +
C           INDSAV(MAXSIN)                                                     +
C           ABVEC(2*MAXSIN)                                                    +
C           XAUX(0:MAXCOL) FOR RHO-ALGORITHM AND THE GENERALIZATIONS OF THE    +
C                          EPSILON-ALGORITHM.                                  +
C           XAUX(0:1)      FOR THE EPSILON-ALGORITHM.                          +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE EPSRHO (EPSVEC, MAXCOL, INIT, EPS, EPSINI, NDIGIT,
     *       MAXSIN, NSING, INDSAV, ABVEC, XAUX, XINIT, IALG, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IALG, MAXCOL, MAXSIN, NDIGIT, NSING
      INTEGER INDSAV(1)
      DOUBLE PRECISION EPS, EPSINI, XINIT
      DOUBLE PRECISION EPSVEC(0:1), XAUX(0:1), ABVEC(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER IND, INDM, INDS, INDXL, INDXU, LISTBEG, LISTEND, MAXCP1
      DOUBLE PRECISION ANDIG, AUX0, AUX1, AUX2, COEFF, DTMP, RD
      LOGICAL*1 IAC, IBC, IEC, LOCINS, NEWINS
      SAVE LISTBEG, LISTEND, INDM, MAXCP1,
     *     LOCINS, NEWINS, IEC, IAC, IBC, 
     *     ANDIG
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0                                                           
         INIT   = 1
         NSING  = 0
         ANDIG  = 10.0D0 ** (-FLOAT(NDIGIT))
         LOCINS = .FALSE.
         NEWINS = .FALSE.
         IEC    = .FALSE.
         IAC    = .FALSE.
         IBC    = .FALSE.
         INDM   = 0
C
C                        ... IF MAXCOL IS ODD IT IS CHANGED TO
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
         IF (IALG .LT. 0 .OR. IALG .GT. 3) IALG = 0
         IF (IALG .NE. 0) XAUX(0) = XINIT
         EPSVEC(0) = EPSINI
         MAXCP1    = MAXCOL + 1
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         IF (IALG .NE. 0) XAUX(0) = XINIT
         EPSVEC(0) = EPSINI
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE BACKWARD DIAGONAL TO 
C                            BE COMPUTED AND THE LOWER INDEX m OF THE 
C                                                          (0)
C                            LAST VALUE OF THIS DIAGONAL eps    TO BE 
C                                                           m
C                            COMPUTED IN THE CALL.
C
      INDM = INDM + 1 
C
C                        ... IF INDM .GT. MAXCOL THEN THE ELEMENT 
C                               (NUMBER_OF_CALLS-MAXCOL-1)
C                            eps                          IS COMPUTED.
C                               MAXCOL
C
      IND = MIN0(INDM,MAXCOL)
C
C                        ... IF THERE IS NO LOCAL INSTABILITY PRESENT,
C                            RESET THE INDEXES LISTBEG AND LISTEND OF
C                            THE VECTOR INDSAV.
C
      IF (.NOT. LOCINS) THEN
         LISTBEG = 1
         LISTEND = 0
      END IF
C
C                        ... STORE THE XINIT VALUE FOR THE RHO AND 
C                            THE GENERALIZATIONS OF THE EPSILON-ALGORITHM.
C
      IF (IALG .NE. 0) THEN
         INDXL = MOD(INDM,MAXCP1)
         XAUX(INDXL) = XINIT
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
      AUX1   = EPSINI
C  
C                        ... SET THE FIXED VALUE OF COEFF ACCORDING 
C                            TO THE DESIRED ALGORITHM.
C
      IF (IALG .EQ. 0) COEFF = 1.0D0
      IF (IALG .EQ. 2) THEN
          INDXU = MOD(INDM-1,MAXCP1)
          COEFF = XAUX(INDXL)-XAUX(INDXU) 
      END IF
C  
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THE ASCENDING DIAGONAL.
C
      DO 10 INDS = 0, IND - 1
C  
C                        ... CHOICE FOR THE VALUE OF COEFF DEPENDING
C                            ON THE VALUE OF INDS.
C
         IF (IALG .EQ. 1 .OR. IALG .EQ. 3) THEN
            INDXL = INDM
            INDXU = MOD(INDM-INDS-1,MAXCP1)
            IF (IALG .EQ. 1) INDXL = INDXL - INDS
            INDXL = MOD(INDXL,MAXCP1)
            COEFF = XAUX(INDXL)-XAUX(INDXU) 
         END IF
C  
C                        ... DENOMINATOR OF THE ALGORITHM FORMULA.
C
         AUX0 = AUX1 - EPSVEC(INDS)
C  
C                        ... CONTROL FOR A NEW POINT OF INSTABILITY.
C
         RD = DMAX1(DABS(AUX1),DABS(EPSVEC(INDS)))
         IF (DABS(EPSVEC(INDS)) .GT. EPS .AND. .NOT.IEC  .AND.
     *       (RD .LT. EPS .OR. DABS(AUX0/RD) .LT. ANDIG) .AND. 
     *       INDS .LT. IND-1 .AND. INDS .LT. MAXCOL-2) THEN
            NEWINS  = .TRUE.
            NSING   = NSING+1
            LISTEND = LISTEND+1
            IF (LISTEND .GT. MAXSIN) THEN
               IER = 1100
               RETURN
            END IF
         END IF 
C
         IF (LOCINS .OR. NEWINS) THEN
C
C                        ... WHEN THERE IS A LOCAL INSTABILITY PRESENT
C                            OR A NEW POINT OF INSTABILITY WE USE
C                            THE SINGULAR RULE OF THE ALGORITHM.
C
            IF (NEWINS) THEN
C   
C                        ... THERE IS A NEW POINT OF INSTABILITY ...
C
               IF (LOCINS) THEN
C  
C                        ... AND THERE IS A LOCAL INSTABILITY PRESENT.
C
C  
C                        ... IF INDS COINCIDES WITH THE FIRST ELEMENT
C                            STORED IN INDSAV THEN THERE IS A NON ISOLATED 
C                            SINGULARITY AND IT IS IMPOSSIBLE TO CONTINUE.
C
                  IF (INDS .EQ. INDSAV(LISTBEG)) THEN
                     IER = 1200
                     RETURN
                  END IF
               END IF
C  
C                        ... LOCAL INSTABILITY PRESENT IS TRUE.
C
               LOCINS = .TRUE.
C
C                        ... WE MUST COMPUTE A!
C
               IAC    = .TRUE.
C  
C                        ... NEW POINT OF INSTABILITY TREATED!
C
               NEWINS = .FALSE.
C
C                        ... ADD THE CURRENT INDS TO THE END OF INDSAV.
C
               INDSAV(LISTEND) = INDS
            END IF
C   
C                        ... WHEN THE CONTROL ON A NEW POINT OF INSTABILITY
C                            IS MADE, THE CONTROL FOR THE COMPUTATION OF E
C                            IS PERFORMED.
C
            IF (IEC) THEN
C
C                        ... WE MUST COMPUTE E AND STORE IT IN AUX0.
C
               DTMP  = DTMP + ABVEC(LISTBEG+MAXSIN) - ABVEC(LISTBEG)
               IF (DABS(EPSVEC(INDS-1)) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               AUX0 = DTMP / (COEFF + DTMP / EPSVEC(INDS-1))
C  
C                        ... E COMPUTED!
C
               IEC     = .FALSE.
C
C                        ... INCREASE THE POINTER ON THE VECTOR INDSAV.
C
               LISTBEG = LISTBEG + 1
C
C                        ... IF, IN THE VECTOR INDSAV, ALL THE STORED
C                            INDEXES HAVE BEEN CONSIDERED, THERE IS NO
C                            LOCAL INSTABILITY PRESENT.
C
               IF (LISTBEG .GT. LISTEND) LOCINS = .FALSE.
C
            ELSE
C
C                        ... IF WE DON'T COMPUTE E THE NORMAL RULE IS 
C                            CONSIDERED.
C
               IF (DABS(AUX0) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
               AUX0 = COEFF / AUX0 
               IF (INDS .NE. 0) AUX0 = AUX0 + EPSVEC(INDS-1)
            END IF
C
            IF (LOCINS) THEN
C   
C                        ... THERE IS A LOCAL INSTABILITY PRESENT ...
C
               IF (IAC) THEN
C
C                        ... AND WE MUST COMPUTE AND STORE THE VALUE 
C                            A = COEFF * W / (1 - W/C).
C
                  IF (INDS .NE. 0) THEN
                     IF (DABS(AUX0) .LT. EPS) THEN
                        IER = 100
                        RETURN
                     END IF
                     ABVEC(LISTEND) = COEFF * EPSVEC(INDS-1) 
     *                       / (1.0D0 - EPSVEC(INDS-1) / AUX0)
                  ELSE
                     ABVEC(LISTEND) = 0.0D0
                  END IF
C  
C                        ... A COMPUTED!
C
                  IAC = .FALSE.
C  
C                        ... WE MUST COMPUTE ALSO B!
C
                  IBC = .TRUE.
C
C                        ... IF WE DON'T COMPUTE A ...
C
               ELSE IF (IBC) THEN
C
C                        ... BUT WE MUST COMPUTE AND STORE THE VALUE 
C                            B = COEFF * N / (1 - N/C).
C
                  IF (DABS(AUX1) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  ABVEC(LISTEND+MAXSIN) = COEFF * EPSVEC(INDS)
     *                    / (1.0D0 - EPSVEC(INDS) / AUX1)
C  
C                        ... B COMPUTED!
C
                  IBC = .FALSE.
C
C                        ... IF WE DON'T COMPUTE A AND WE DON'T COMPUTE B ...
C
               ELSE IF (INDS .NE. 0 .AND. INDS .EQ. (INDSAV(LISTBEG)+1))
     *                 THEN
C
C                        ... AND THE CURRENT INDEX INDS IS THE NEXT
C                            INDEX OF INSTABILITY IN INDSAV WE COMPUTE 
C                            D = COEFF * S / (1 - S/C) ...
C
                  IF (DABS(EPSVEC(INDS)) .LT. EPS) THEN
                     IER = 100
                     RETURN
                  END IF
                  DTMP = COEFF * AUX1 / (1.0D0 - AUX1 / EPSVEC(INDS))
C  
C                        ... AND WE MUST COMPUTE E!
C
                  IEC = .TRUE.
               END IF
            END IF 
         ELSE
C
C                        ... WHEN THERE IS NO LOCAL INSTABILITY PRESENT
C                            NOR A NEW POINT OF INSTABILITY WE USE
C                            THE NORMAL RULE OF THE ALGORITHM.
C  
            IF (DABS(AUX0) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            AUX0 = COEFF / AUX0 
            IF (INDS .NE. 0) AUX0 = AUX0 + EPSVEC(INDS-1)
         END IF
C  
C                        ... SHIFT THE RHOMBUS.
C
         IF (INDS .NE. 0) EPSVEC(INDS-1) = AUX2
         AUX2 = AUX1
         AUX1 = AUX0
C  
C                        ... END LOOP FOR INDS
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES OF THE DIAGONAL.
C
      EPSVEC(IND-1) = AUX2
      EPSVEC(IND)   = AUX1
C  
C                        ... SET THE RESULT VALUE.
C
      IF (MOD(IND,2) .EQ. 0) THEN
         EPSINI = EPSVEC(IND)
      ELSE
         EPSINI = EPSVEC(IND-1)
      END IF
      RETURN
      END
