C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - EPSTOP                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE SECOND TOPOLOGICAL ESPILON-ALGORITHM.  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL EPSTOP (EPSMAT, MAXCOL, INIT, EPS, EPSINI, Y, IR,             +
C              IP, WG, IER)                                                    +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           EPSMAT - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSIONS OF EPSMAT MUST BE (IP,0:IE), WITH IE GREATER   +
C                    OR EQUAL TO MAXCOL (IF MAXCOL IS EVEN) OR TO (MAXCOL-1)   +
C                    (IF MAXCOL IS ODD). THIS MATRIX CONTAINS, AFTER THE k-TH  +
C                    CALL THE LAST COMPUTED BACKWARD DIAGONAL OF THE EPSILON   +
C                    SCHEME STORED BY COLUMNS:                                 +
C                                                                              +
C                                          (k-1)   (k-2)        (0)            +
C                     - IF k <= MAXCOL+1 eps   , eps    ,..., eps              +
C                                           0       1            k-1           +
C                                                                              +
C                                          (k-1)   (k-2)        (k-1-MAXCOL)   +
C                     - IF k > MAXCOL+1  eps   , eps    ,..., eps              +
C                                           0       1            MAXCOL        +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE EPSILON SCHEME THAT THE USER WANTS TO COMPUTE      +
C                                           (n)                                +
C                    (THE LOWER INDEX OF eps   ). IF MAXCOL IS ODD THE         +
C                                            m                                 +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           EPSINI - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUE       +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                       (0)                                                    +
C                    eps            IF  k <= MAXCOL+1, k ODD                   +
C                       k-1                                                    +
C                                                                              +
C                       (1)                                                    +
C                    eps            IF  k <  MAXCOL+1, k EVEN                  +
C                       k-2                                                    +
C                                                                              +
C                       (k-1-MAXCOL)                                           +
C                    eps            IF  k >  MAXCOL+1                          +
C                       MAXCOL                                                 +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                      (0)   (1)   (0)   (1)       (0)      (1)      (2)       +
C                    eps , eps , eps , eps ,..., eps    , eps    , eps   , ... +
C                       0     0     2     2         MAXCOL   MAXCOL   MAXCOL   +
C                                                                              +
C           Y      - INPUT REAL VECTOR WHICH MUST CONTAIN THE ARBITRARY        +
C                    VECTOR y.                                                 +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX EPSMAT.                                            +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF            +
C                    ELEMENTS OF THE VECTORS EPSINI AND Y).                    +
C                                                                              +
C           WG     - WORKING REAL VECTOR OF DIMENSION 4*IP.                    +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  DOUBLE PRECISION FUNCTION  INNERP                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE MATRIX EPSMAT AND THE VECTOR WG MUST NOT   +
C          BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE        +
C          SUBROUTINE.                                                         +
C                                    (0)                                       +
C       -  TO OBTAIN THE VALUE OF eps   (2*J <= MAXCOL) THE USER MUST CALL     +
C                                    2*J                                       +
C          THIS SUBROUTINE (2*J+1) TIMES.                                      +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=IP) ARE:                            +
C           EPSMAT(IP,0:MAXCOL)                                                +
C           EPSINI(IP)                                                         +
C           Y(IP)                                                              +
C           WG(4*IP)                                                           +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE EPSTOP (EPSMAT, MAXCOL, INIT, EPS, EPSINI, Y, IR, 
     *       IP, WG, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IP, IR, MAXCOL
      DOUBLE PRECISION EPS
      DOUBLE PRECISION EPSMAT(IR,0:1), EPSINI(1), Y(1), WG(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION INNERP
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER II, IP1, IP2, INDM, INDS
      DOUBLE PRECISION RD
C
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
         IER    = 0                                                           
         INIT   = 1
         INDM   = 0
C
C                        ... IF MAXCOL IS ODD IT IS MODIFIED TO 
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
C
         DO 10 II = 1, IP
            EPSMAT(II,0) = EPSINI(II)
   10    CONTINUE
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         DO 20 II = 1, IP
            EPSMAT(II,0) = EPSINI(II)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE BACKWARD DIAGONAL TO 
C                            BE COMPUTED AND THE LOWER INDEX m OF THE 
C                                                          (0)
C                            LAST VALUE OF THIS DIAGONAL eps    TO BE 
C                                                           m
C                            COMPUTED IN THE CALL.
C
      INDM = INDM + 1 
C
C                        ... IF INDM .GT. MAXCOL THEN THE ELEMENT 
C                               (NUMBER_OF_CALLS-MAXCOL+1)
C                            eps                          IS COMPUTED.
C                               MAXCOL
C
      IF (INDM .GT. MAXCOL) INDM = MAXCOL
C
C                        ... INITIALIZATION FOR THE AUXILIARY VECTOR.
C
      DO 30 II = 1, IP
         WG(IP+II) = EPSINI(II)
   30 CONTINUE
C
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THE ASCENDING DIAGONAL.
C
      DO 100 INDS = 0, INDM - 1
         IF (MOD(INDS,2) .EQ. 0) THEN
C  
C                        ... COMPUTE A TERM OF AN ODD COLUMN.
C
            DO 40 II = 1, IP
               WG(3*IP+II) = WG(IP+II) - EPSMAT(II,INDS)
   40       CONTINUE
            RD = INNERP(Y, 1, WG, 4, IP, IP)
            IF (DABS(RD) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            DO 50 II = 1, IP
               WG(II) = Y(II) / RD
               IF (INDS .NE. 0) WG(II) = WG(II) + EPSMAT(II,INDS-1)
   50       CONTINUE
C  
C                        ... SHIFT THE RHOMBUS.
C
            DO 60 II = 1, IP
               IP1 = IP + II
               IP2 = 2*IP + II
               IF (INDS .NE. 0) EPSMAT(II,INDS-1) = WG(IP2)
               WG(IP2) = WG(IP1)
               WG(IP1) = WG(II)
   60       CONTINUE
            IF (INDS .NE. INDM-1) THEN
C  
C                        ... COMPUTE THE DENOMINATOR FOR THE TERM
C                            OF THE NEXT EVEN COLUMN.
C
               DO 70 II = 1, IP
                  WG(II) = WG(II) - EPSMAT(II,INDS+1)
   70          CONTINUE
               RD = INNERP(WG, 1, WG, 4, IP, IP)
               IF (DABS(RD) .LT. EPS) THEN
                  IER = 100
                  RETURN
               END IF
            END IF
         ELSE
C  
C                        ... COMPUTE A TERM OF AN EVEN COLUMN.
C
            DO 80 II = 1, IP
               WG(II) = WG(3*IP+II) / RD + EPSMAT(II,INDS-1)
   80       CONTINUE
C  
C                        ... SHIFT THE RHOMBUS.
C
            DO 90 II = 1, IP
               IP1 = IP + II
               IP2 = 2*IP + II
               EPSMAT(II,INDS-1) = WG(IP2)
               WG(IP2) = WG(IP1)
               WG(IP1) = WG(II)
   90       CONTINUE
         END IF
C                        ... END LOOP FOR INDS.
C
  100 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES OF THE DIAGONAL.
C
      DO 110 II = 1, IP
         EPSMAT(II,INDM-1) = WG(2*IP+II)
         EPSMAT(II,INDM)   = WG(IP+II)
C  
C                        ... SET THE RESULT VALUE.
C
         IF (MOD(INDM,2) .EQ. 0) THEN
            EPSINI(II) = EPSMAT(II,INDM)
         ELSE
            EPSINI(II) = EPSMAT(II,INDM-1)
         END IF
  110 CONTINUE
      RETURN
      END
