C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - IDELTA                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE ITERATED DELTA 2 PROCESS               +
C                                                                              +
C   USAGE:                                                                     +
C           CALL IDELTA (XVEC, MAXCOL, INIT, EPS, XINIT, IER)                  +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           XVEC   - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF XVEC MUST BE (0:IE), WITH IE GREATER         +
C                    OR EQUAL TO 2*MAXCOL. THIS VECTOR CONTAINS, AFTER         +
C                    THE k-TH CALL:                                            +
C                                                                              +
C                     - IF k = 1                                               +
C                                                                              +
C                        (0)                                                   +
C                       x                                                      +
C                        0                                                     +
C                                                                              +
C                     - IF 1 < k < 2*MAXCOL AND k ODD                          +
C                                                                              +
C                        (k-2) (k-1) (k-4) (k-3)    (2)        (0)             +
C                       x   , x   , x   , x  ,..., x      ,   x                +
C                        0     0     1     1        (k-1)/2-1  (k-1)/2         +
C                                                                              +
C                     - IF 1 < k <= 2*MAXCOL AND k EVEN                        +
C                                                                              +
C                        (k-2) (k-1) (k-4) (k-3)    (3)    (0)     (1)         +
C                       x   , x   , x   , x  ,..., x    , x    ,  x            +
C                        0     0     1     1        k/2-2  k/2-1   k/2-1       +
C                                                                              +
C                     - IF k > 2*MAXCOL                                        +
C                                                                              +
C                        (k-2) (k-1) (k-4) (k-3)                               +
C                       x   , x   , x   , x        ,...,                       +
C                        0     0     1     1                                   +
C                                                                              +
C                              (k-2*MAXCOL)   (k-2*MAXCOL+1)   (k-2*MAXCOL-1)  +
C                             x            , x              , x                +
C                              MAXCOL-1       MAXCOL-1         MAXCOL          +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF LAST THE COLUMN THAT    +
C                                                                   (j)        +
C                    THE USER WANTS TO COMPUTE (THE LOWER INDEX OF x   ).      +
C                                                                   m          +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           XINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    XINIT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE      +
C                                                                              +
C                     (0)                                                      +
C                    x            IF  k = 1                                    +
C                     0                                                        +
C                                                                              +
C                     (1)                                                      +
C                    x            IF  1 < k <= 2*MAXCOL, k EVEN                +
C                     k/2-1                                                    +
C                                                                              +
C                     (0)                                                      +
C                    x            IF  1 < k <  2*MAXCOL, k ODD                 +
C                     (k-1)/2                                                  +
C                                                                              +
C                     (k-2*MAXCOL-1)                                           +
C                    x            IF  k > 2*MAXCOL                             +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)  (1)  (0)  (1)  (0)      (0)     (1)     (2)         +
C                    x ,  x  , x  , x  , x  ,..., x     , x     , x     , ...  +
C                     0    0    1    1    2        MAXCOL  MAXCOL  MAXCOL      +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST       +
C          BE IN DOUBLE PRECISION AND THE VECTOR XVEC MUST NOT BE MODIFIED     +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C                                   (0)                                        +
C       -  TO OBTAIN THE VALUE OF  x   (J <= MAXCOL) THE USER MUST CALL THIS   +
C                                   J                                          +
C          SUBROUTINE (2*J+1) TIMES.                                           +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSION FOR THE VECTOR       +
C          ARGUMENT IS:                                                        +
C           XVEC(0:2*MAXCOL)                                                   +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE IDELTA (XVEC, MAXCOL, INIT, EPS, XINIT, IER)       
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION EPS, XINIT
      DOUBLE PRECISION XVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER IND, INDM, INDS, INDP1, MAXC2
      DOUBLE PRECISION AUX0, AUX1
      SAVE INDM, MAXC2
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER     = 0
         INIT    = 1
         INDM    = 0
         XVEC(0) = XINIT
         MAXC2   = 2*MAXCOL 
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C
      IF (MAXCOL .EQ. 0) THEN
         XVEC(0) = XINIT
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1).
C
      INDM = INDM + 1 
C  
C                        ... SET THE RESULT VALUE IF INDM = 1
C
      IF (INDM .EQ. 1) THEN
         XVEC(1) = XINIT
         RETURN
      END IF 
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                                                    INDM
C                            AUX1 WITH THE VALUE OF x      = S     .
C                                                    0        INDM
C
      AUX1   = XINIT
C
C                        ... IF INDM > 2*MAXCOL THEN THE ELEMENT
C                             (NUMBER_OF_CALLS-2*MAXCOL-1)
C                            x                            IS COMPUTED.
C                             MAXCOL
C                            
      IF (INDM .LE. MAXC2) THEN
         IND = 2 * INT(INDM/2)
      ELSE
         IND = MAXC2
      END IF
C  
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS.
C
      DO 10 INDS = 0, IND - 2, 2
         INDP1 = INDS + 1
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENT.
C
         AUX0 = AUX1 - 2.0D0 * XVEC(INDP1) + XVEC(INDS)
         IF (DABS(AUX0) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         AUX0 = AUX1 - (AUX1 - XVEC(INDP1))**2 / AUX0
C  
C                        ... SHIFT THE VALUES OF THE VARIABLES.
C
         XVEC(INDS)  = XVEC(INDP1)
         XVEC(INDP1) = AUX1
         AUX1 = AUX0
C  
C                        ... END LOOP FOR INDS.
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES.
C
      IF (MOD(INDM,2) .NE. 0 .AND. INDM .LT. MAXC2) THEN
         INDP1 = IND + 1
      ELSE
         INDP1 = IND
      END IF
      XVEC(INDP1) = AUX1
C  
C                        ... SET THE RESULT VALUE.
C
      XINIT = XVEC(INDP1)
      RETURN
      END
