C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - LEVINT                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SCALAR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE LEVIN'S TRANSFORMATIONS.               +
C                                                                              +
C   USAGE:                                                                     +
C           CALL LEVINT (VECN, VECD, MAXCOL, INIT, EPS, LINIT,                 +
C              GINIT, B, ITRAN, IER)                                           +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           VECN   - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF VECN MUST BE (0:IE), WITH IE GREATER OR      +
C                    EQUAL TO MAXCOL. THIS VECTOR CONTAINS, AFTER THE k-TH     +
C                    CALL THE LAST COMPUTED ROW OF THE N-ARRAY:                +
C                                                                              +
C                                             (0)                              +
C                     - IF k = 1             N           (ITRAN = 0, 1, 2)     +
C                                             0                                +
C                                            UNDEFINED   (ITRAN = 3)           +
C                                                                              +
C                                             (k-1-J)  (k-2-J)       (0)       +
C                     - IF 1< k <= MAXCOL+1  N      , N      ,..., N           +
C                                             0        1            k-1-J      +
C                                                                              +
C                                             (k-1-J) (k-2-J)   (k-1-MAXCOL-J) +
C                     - IF    k > MAXCOL+1   N     , N   ,..., N               +
C                                             0       1         MAXCOL         +
C                                                                              +
C                       WITH J = 0   (ITRAN = 0, 1, 2)                         +
C                       WITH J = 1   (ITRAN = 3)                               +
C                                                                              +
C           VECD   - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF VECD MUST BE (0:IE), WITH IE GREATER OR      +
C                    EQUAL TO MAXCOL. THIS VECTOR CONTAINS, AFTER THE k-TH     +
C                    CALL THE LAST COMPUTED ROW OF THE D-ARRAY:                +
C                                                                              +
C                                             (0)                              +
C                     - IF k = 1             D           (ITRAN = 0, 1, 2)     +
C                                             0                                +
C                                            UNDEFINED   (ITRAN = 3)           +
C                                                                              +
C                                             (k-1-J)  (k-2-J)       (0)       +
C                     - IF 1< k <= MAXCOL+1  D      , D      ,..., D           +
C                                             0        1            k-1-J      +
C                                                                              +
C                                             (k-1-J) (k-2-J)   (k-1-MAXCOL-J) +
C                     - IF    k > MAXCOL+1   D     , D   ,..., D               +
C                                             0       1         MAXCOL         +
C                                                                              +
C                       WITH J = 0   (ITRAN = 0, 1, 2)                         +
C                       WITH J = 1   (ITRAN = 3)                               +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN         +
C                    OF THE L-ARRAY THAT THE USER WANTS TO COMPUTE             +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF L   ).                                +
C                                         k                                    +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           LINIT  - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF S    BEFORE THE k-TH CALL OF THE SUBROUTINE.           +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                                                                              +
C                     (0)                                                      +
C                    L            IF k = 1              (ITRAN = 0, 1, 2, 3)   +
C                     0                                                        +
C                                                                              +
C                     (0)                                                      +
C                    L            IF  1 < k <= MAXCOL+1                        +
C                     k-1-J                                                    +
C                                                                              +
C                     (k-1-J-MAXCOL)                                           +
C                    L            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                       WITH J = 0   (ITRAN = 0, 1, 2)                         +
C                       WITH J = 1   (ITRAN = 3)                               +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)   (0)   (0)         (0)      (1)      (2)            +
C                    L   , L   , L   , ... , L      , L      , L      , ...    +
C                     0     1     2           MAXCOL   MAXCOL   MAXCOL         +
C                       IF ITRAN = 0, 1, 2                                     +
C                                                                              +
C                     (0)   (0)   (0)   (0)       (0)      (1)      (2)        +
C                    L   , L   , L   , L  ,... , L      , L      , L    , ...  +
C                     0     0     1     2         MAXCOL   MAXCOL   MAXCOL     +
C                       IF ITRAN = 3                                           +
C                                                                              +
C           GINIT  - INPUT REAL VALUE. IT MUST CONTAIN THE VALUE g(k-1) OF     +
C                    THE AUXILIARY SEQUENCE, BEFORE THE  k-TH CALL OF          +
C                    THE SUBROUTINE. VALID ONLY FOR A GENERAL AUXILIARY        +
C                    SEQUENCE (ITRAN=0). IN THE OTHER CASES ITS VALUE IS       +
C                    NOT USED.                                                 +
C                                                                              +
C           B      - INPUT REAL VALUE. THIS CONSTANT MUST BE ANY NON-ZERO      +
C                    REAL NUMBER, BUT NOT A NEGATIVE INTEGER. THIS VALUE       +
c                    IS USED IN THE COMPUTATION OF THE TRANSFORMATION.         +
C                    IF B < 0 AND DABS(AINT(B)-B) .LT. EPS, THEN B IS          +
C                    CONSIDERED TO BE A NEGATIVE INTEGER CONSTANT.             +
C                                                                              +
C           ITRAN  - INPUT/OUTPUT INTEGER FLAG.                                +
C                    IF ITRAN .EQ. 0 THE SUBROUTINE PERFORMS THE TRANSFORM     +
C                                    WITH A GENERAL AUXILIARY SEQUENCE.        +
C                    IF ITRAN .EQ. 1 THE SUBROUTINE PERFORMS THE u-TRANSFORM   +
C                    IF ITRAN .EQ. 2 THE SUBROUTINE PERFORMS THE t-TRANSFORM   +
C                    IF ITRAN .EQ. 3 THE SUBROUTINE PERFORMS THE v-TRANSFORM   +
C                    IF ITRAN .LT. 0 .OR. ITRAN .GT. 3 THE SUBROUTINE PUTS     +
C                    ITRAN TO ZERO AND THUS IT PERFORMS THE TRANSFORM WITH THE +
C                    VALUE OF GINIT.                                           +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 500     CALL OF THE SUBROUTINE WITH A VALUE OF      +
C                                  B NON ADMITTED.                             +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION AND THE VECTORS VECN AND VECD MUST NOT BE       +
C          MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE           +
C          SUBROUTINE.                                                         +
C       -  IF MAXCOL = 0 THE SUBROUTINE RETURNS IN LINIT ALWAYS THE SEQUENCE   +
C          S , S , S , S , ... AND THE VALUES OF VECN AND VECD ARE UNDEFINED.  +
C           0   1   2   3                                                      +
C       -  THE VALUES OF THE ROWS OF THE L-ARRAY CAN BE OBTAINED PERFORMING    +
C          THE RATIO BETWEEN THE COMPONENTS WITH THE SAME INDEX OF THE VECTORS +
C          VECN AND VECD.                                                      +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF L   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                                           +
C          SUBROUTINE J+1 TIMES (ITRAN = 0, 1, 2) OR J+2 TIMES (ITRAN = 3).    +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           VECN(0:MAXCOL)                                                     +
C           VECD(0:MAXCOL)                                                     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE LEVINT (VECN, VECD, MAXCOL, INIT, EPS, LINIT, 
     *       GINIT, B, ITRAN, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, ITRAN, MAXCOL
      DOUBLE PRECISION B, GINIT, LINIT, EPS
      DOUBLE PRECISION VECD(0:1), VECN(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES
C
      DOUBLE PRECISION COEFF, COEF1, COEF2, LPREC, LPREC1, 
     *                 TMPN0, TMPD0, TMPN1, TMPD1
      INTEGER IND, INDM, INDS
      SAVE INDM, LPREC, LPREC1
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER      = 0
         INIT     = 1
C
C                        ... CONTROL FOR THE VALUE OF B
C
         COEFF = AINT(B)
         IF (B .LE. 0.0D0 .AND. DABS(COEFF-B) .LT. EPS) THEN
            IER = 500
            RETURN
         END IF
C
         IF (MAXCOL .EQ. 0) RETURN
         IF (ITRAN .LT. 0 .OR. ITRAN .GT. 3) ITRAN = 0
         IF (ITRAN .EQ. 1) GINIT = B * LINIT
         IF (ITRAN .EQ. 2) GINIT = LINIT
         IF (ITRAN .NE. 3) THEN
            INDM   = 0
            IF (DABS(GINIT) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            VECN(0) = LINIT/GINIT
            VECD(0) = 1.0D0/GINIT
         ELSE
            INDM   = -1
         END IF
         LPREC = LINIT
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) RETURN
C
C                        ... INDM REPRESENTS THE 
C                            - NUMBER_OF_CALLS - 1 IF ITRAN = 0, 1, 2
C                            - NUMBER_OF_CALLS - 2 IF ITRAN = 3.
C
      INDM = INDM + 1
C
C                        ... COMPUTE THE VALUE OF GINIT
C
      IF (ITRAN .EQ. 1 .OR. ITRAN .EQ. 2) THEN
         GINIT = (LINIT - LPREC)
         IF (ITRAN .EQ. 1) GINIT = (INDM + B) * GINIT
         LPREC = LINIT
      END IF
C
      IF (ITRAN .EQ. 3) THEN
         IF (INDM .EQ. 0) THEN
            COEFF  = LINIT - 2*LPREC
            IF (DABS(COEFF) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            GINIT   = -LPREC - LPREC**2 / COEFF 
            IF (DABS(GINIT) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            VECN(0) = LPREC/GINIT
            VECD(0) = 1.0D0/GINIT
         ELSE
            COEFF  = LINIT - 2*LPREC1 + LPREC
            IF (DABS(COEFF) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            GINIT  = -(LPREC1 - LPREC) * (LINIT - LPREC1) / COEFF
            LPREC  = LPREC1
         END IF
         LPREC1 = LINIT
         LINIT  = LPREC
         IF (INDM .EQ. 0) RETURN
      END IF
C
C                                                  (INDM)     (INDM)
C                        ... COMPUTE THE ELEMENTS N      AND D 
C                                                  0          0
C
      IF (DABS(GINIT) .LT. EPS) THEN
         IER = 100
         RETURN
      END IF
      TMPN1  = LINIT/GINIT
      TMPD1  = 1.0D0/GINIT
C
C                                                  (INDM-1)     (INDM-1) 
C                        ... COMPUTE THE ELEMENTS N        AND D 
C                                                  1            1
C
      TMPN0   = TMPN1 - VECN(0)
      TMPD0   = TMPD1 - VECD(0)
      VECN(0) = TMPN1
      VECD(0) = TMPD1
C
      IF (INDM .EQ. 1) THEN
         IF (DABS(TMPD0) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         LINIT    = TMPN0 / TMPD0
         VECN(1)  = TMPN0
         VECD(1)  = TMPD0
         RETURN
      END IF
C
      TMPN1    = TMPN0
      TMPD1    = TMPD0
C
C                        ... COMPUTE THE OTHER ELEMENTS OF THE ROW
C                            IN THE N AND D ARRAYS
C
      IND    = MIN0(INDM-1,MAXCOL-1)
      COEF1  = DBLE(INDM-1) + B
      COEF2  = DBLE(INDM) + B
      DO 10 INDS = 1, IND
         COEFF  = (DBLE(INDM-1-INDS) + B)* COEF1**(INDS-1) / COEF2**INDS
         TMPN0      = TMPN1 - VECN(INDS) * COEFF
         TMPD0      = TMPD1 - VECD(INDS) * COEFF
         VECN(INDS) = TMPN1
         VECD(INDS) = TMPD1
         IF (INDS .NE. IND) THEN
            TMPN1   = TMPN0
            TMPD1   = TMPD0
         END IF
   10 CONTINUE
C
C                        ... SAVE FOR THE FINAL VALUES.
C
      VECN(IND+1) = TMPN0
      VECD(IND+1) = TMPD0
      IF (DABS(TMPD0) .LT. EPS) THEN
         IER = 100
         RETURN
      END IF
C
C                        ... SET THE RESULT VALUE.
C
      LINIT  = TMPN0 / TMPD0
      RETURN
      END
