C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - OMEGA                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE w-ALGORITHM                            +
C                                                                              +
C   USAGE:                                                                     +
C           CALL OMEGA (OMEVEC, MAXCOL, INIT, EPS, OMEINI, IER)                +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           OMEVEC - OUTPUT REAL VECTOR. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF OMEVEC MUST BE (0:IE), WITH IE GREATER       +
C                    OR EQUAL TO MAXCOL (IF MAXCOL IS EVEN) OR TO (MAXCOL-1)   +
C                    (IF MAXCOL IS ODD). THIS VECTOR CONTAINS, AFTER THE k-TH  +
C                    CALL, THE LAST COMPUTED BACKWARD DIAGONAL OF THE SCHEME:  +
C                                                                              +
C                                         (k-1)   (k-2)        (0)             +
C                     - IF k <= MAXCOL+1 w     , w      ,..., w                +
C                                         0       1            k-1             +
C                                                                              +
C                                         (k-1)   (k-2)        (k-1-MAXCOL)    +
C                     - IF k >  MAXCOL+1 w     , w      ,..., w                +
C                                         0       1            MAXCOL          +
C                                                                              +
C           MAXCOL - INPUT/OUTPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN  +
C                    OF THE EPSILON SCHEME THAT THE USER WANTS TO COMPUTE      +
C                                         (n)                                  +
C                    (THE LOWER INDEX OF w   ). IF MAXCOL IS ODD THE           +
C                                         m                                    +
C                    SUBROUTINE CHANGES ITS VALUE TO (MAXCOL-1).               +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           OMEINI - INPUT/OUTPUT REAL VALUE. IT MUST CONTAIN THE VALUE        +
C                    OF a   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    IT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE         +
C                                                                              +
C                     (0)                                                      +
C                    w            IF  k <= MAXCOL+1, k ODD                     +
C                     k-1                                                      +
C                                                                              +
C                     (1)                                                      +
C                    w            IF  k <  MAXCOL+1, k EVEN                    +
C                     k-2                                                      +
C                                                                              +
C                     (k-1-MAXCOL)                                             +
C                    w            IF  k >  MAXCOL+1                            +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                      (0)   (1)   (0)   (1)       (0)      (1)      (2)       +
C                    w    , w   , w   , w   ,..., w      , w      , w    , ... +
C                     0      0     2     2         MAXCOL   MAXCOL   MAXCOL    +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION AND THE VECTOR OMEVEC MUST NOT BE MODIFIED      +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF w   (2*J <= MAXCOL) THE USER MUST CALL THIS  +
C                                  2*J                                         +
C          SUBROUTINE (2*J+1) TIMES.                                           +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          VECTOR ARGUMENT IS:                                                 +
C           OMEVEC(0:MAXCOL)       IF MAXCOL IS EVEN                           +
C            OR                                                                +
C           OMEVEC(0:MAXCOL-1)     IF MAXCOL IS ODD                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE OMEGA (OMEVEC, MAXCOL, INIT, EPS, OMEINI, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, MAXCOL
      DOUBLE PRECISION EPS, OMEINI
      DOUBLE PRECISION OMEVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER INDM, INDS
      DOUBLE PRECISION AUX0, AUX1, AUX2
C
      SAVE INDM
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IER    = 0                                                           
         INIT   = 1
         INDM   = 0
C
C                        ... IF MAXCOL IS ODD IT IS MODIFIED TO 
C                            (MAXCOL - 1).
C
         IF (MOD(MAXCOL,2) .NE. 0) MAXCOL = MAXCOL-1
C
         OMEVEC(0) = OMEINI
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
      IF (MAXCOL .EQ. 0) THEN
         OMEVEC(0) = OMEINI
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE BACKWARD DIAGONAL TO 
C                            BE COMPUTED AND THE LOWER INDEX m OF THE 
C                                                         (0)
C                            LAST VALUE OF THIS DIAGONAL w    TO BE 
C                                                         m
C                            COMPUTED IN THE CALL.
C
      INDM = INDM + 1 
C
C                        ... IF INDM .GT. MAXCOL THEN THE ELEMENT 
C                             (NUMBER_OF_CALLS-MAXCOL+1)
C                            w                          IS COMPUTED.
C                             MAXCOL
C
      IF (INDM .GT. MAXCOL) INDM = MAXCOL
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE
C                            AUX1.
C
      AUX1 = OMEINI
C
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS
C                            OF THE ASCENDING DIAGONAL.
C
      DO 10 INDS = 0, INDM - 1
         IF (MOD(INDS,2) .EQ. 0) THEN
C  
C                        ... COMPUTE A TERM OF AN ODD COLUMN.
C
            IF (DABS(AUX1) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            AUX0 = OMEVEC(INDS) / AUX1
            IF (INDS .NE. 0) AUX0 = AUX0 + OMEVEC(INDS-1)
C  
C                        ... SHIFT THE RHOMBUS.
C
            IF (INDS .NE. 0) OMEVEC(INDS-1) = AUX2
            AUX2 = AUX1
            AUX1 = AUX0
            IF (INDS .NE. INDM-1) THEN
C  
C                        ... COMPUTE THE DIFFERENCE FOR THE TERM
C                            OF THE NEXT EVEN COLUMN.
C
               AUX0 = OMEVEC(INDS+1) - AUX0
            END IF
         ELSE
C  
C                        ... COMPUTE A TERM OF AN EVEN COLUMN.
C
            AUX0 = OMEVEC(INDS-1) * AUX0
C  
C                        ... SHIFT THE RHOMBUS.
C
            OMEVEC(INDS-1) = AUX2
            AUX2 = AUX1
            AUX1 = AUX0
         END IF
C                        ... END LOOP FOR INDS.
C
   10 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES OF THE DIAGONAL.
C
      OMEVEC(INDM-1) = AUX2
      OMEVEC(INDM)   = AUX1
C  
C                        ... SET THE RESULT VALUE.
C
      IF (MOD(INDM,2) .EQ. 0) THEN
         OMEINI = OMEVEC(INDM)
      ELSE
         OMEINI = OMEVEC(INDM-1)
      END IF
      RETURN
      END
