C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - QBORD                                                +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C                                      (k)     -1                              +
C           COMPUTES THE SOLUTION q = q   = - A   u   RECURSIVELY BY THE       +
C                                  k   k       k   k                           +
C           BORDERING METHOD.                                                  +
C                                                                              +
C   USAGE:                                                                     +
C           CALL QBORD (A, IR, Q, EPS, INIT, IER)                              +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           A      - INPUT REAL SQUARE MATRIX OF DIMENSION (IR,IR).            +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR           +
C                    THE MATRIX A.                                             +
C                                                                              +
C           Q      - OUTPUT REAL VECTOR OF DIMENSION (IR*(IR+3)/2)             +
C                    CONTAINING AN UPPER HESSENBERG MATRIX WHICH, IN THE       +
C                    COLUMN k (ROWS 1,..., k), CONTAINS THE SOLUTION q         +
C                                                                     k        +
C                    AND IN THE ROW (k+1), THE VALUE a  + (v , q ) .           +
C                                                     k     k   k              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE         +
C                    FIRST CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED        +
C                    TO 1 BY THE SUBROUTINE DURING THE FIRST CALL.             +
C                    FOR A NEW APPLICATION OF THE RECURSIVE COMPUTATION        +
C                    INIT MUST SET AGAIN TO ZERO.                              +
C                                                                              +
C           IER    - OUTPUT INDEX WARNING/ERROR.                               +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       - DOUBLE PRECISION FUNCTION INNERC                                     +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C        - ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROGRAM MUST BE      +
C          IN DOUBLE PRECISION.                                                +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE             +
C          MATRIX AND VECTOR ARGUMENTS (IR=NUMBER_OF_CALLS) ARE:               +
C           A(IR,IR)                                                           +
C           Q(IR*(IR+3)/2)                                                     +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       
      SUBROUTINE QBORD (A, IR, Q, EPS, INIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IR
      DOUBLE PRECISION EPS
      DOUBLE PRECISION A(IR,1), Q(1)
C
C                        ... SPECIFICATIONS FOR EXTERNAL FUNCTIONS
C
      DOUBLE PRECISION INNERC
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      DOUBLE PRECISION COEFF
      INTEGER I, IINC, J, K, KINC
      SAVE K
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         INIT = 1
         K    = 1
C                                                              
C                        ... COMPUTES THE SOLUTION q 
C                                                   1
         Q(1) = - A(1,2) / A(1,1)
C
C                        ... COMPUTES THE VALUE OF a  + (v , q ) AND 
C                                                   1     1   1
C                            STORES IT IN Q(2)
C
         Q(2) = A(2,2) + A(2,1) * Q(1)
         IF (DABS(Q(2)) .LT. EPS) IER = 100
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      K    = K + 1
      KINC = (K-1)*(K+2)/2 
C                                      (1)
C                        ... COMPUTES q 
C                                      k
      Q(1+KINC)  = - A(1,K+1) / A(1,1)
C                                                               (k)
C                        ... COMPUTES RECURSIVELY THE SOLUTION q 
C                                                               k
      DO 20 I = 1, K-1
         IINC  = (I-1)*(I+2)/2
         COEFF = (A(I+1,K+1) + INNERC (A, I+1, Q, K, IR, I, 1)) /
     *           Q(I+1+IINC)
         Q(I+1+KINC) = - COEFF 
         DO 10 J = 1, I
            Q(J+KINC) = Q(J+KINC) - COEFF * Q(J+IINC)
   10    CONTINUE
   20 CONTINUE
C
C                        ... COMPUTES THE VALUE OF a  + (v , q ) AND 
C                                                   k     k   k  
C                            STORES IT IN Q(K+1+KINC)
C
      Q(K+1+KINC) = A(K+1,K+1) + INNERC (A, K+1, Q, K, IR, K, 1)
      IF (DABS(Q(K+1+KINC)) .LT. EPS) IER = 100
      RETURN
      END      
