C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - SELECT                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           SELECTION BETWEEN NINE SEQUENCE TRANSFORMATIONS.                   +
C           THE SUBROUTINE USES THE FOLLOWING TRANSFORMATIONS T (I = 1,...,9)  +
C                                                              I               +
C                                                                              +
C            I = 1  EPSILON-ALGORITHM                                          +
C            I = 2  RHO-ALGORITHM WITH x  = n                                  +
C                                       n                                      +
C            I = 3  THETA-ALGORITHM                                            +
C            I = 4  RICHARDSON PROCESS WITH x  = S    -  S                     +
C                                            n    n+1     n                    +
C            I = 5  FIRST GENER. EPSILON-ALGORITHM WITH x  = 1 + 1/(n+1)       +
C                                                        n                     +
C            I = 6  SECOND GENER. EPSILON-ALGORITHM WITH x  = 1 + 1/(n+1)      +
C                                                         n                    +
C            I = 7  ITERATED DELTA2 PROCESS                                    +
C            I = 8  OVERHOLT PROCESS                                           +
C            I = 9  LEVIN'S t-TRANSFORM                                        +
C                                                                              +
C            THE SUBROUTINE STARTS THE SELECTION ONLY AT THE 3-TH CALL BECAUSE +
C            ALMOST ALL THE TRANSFORMATIONS GIVE AN EXTRAPOLATED VALUE ONLY    +
C            AFTER THREE CALLS.                                                +
C                                                                              +
C   USAGE:                                                                     +
C           CALL SELECT (NBC, INIT, EPS, SINIT, ITOR, REST, NDIGIT, IER)       +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           NBC    - INPUT INTEGER. IT REPRESENTS THE MAXIMUM NUMBER OF CALLS  +
C                    OF THIS SUBROUTINE AS GIVEN IN THE CALLING PROGRAM. THE   +
C                    SUBROUTINE USES THIS VALUE TO SET THE DIMENSIONS OF THE   +
C                    ARRAYS USED IN THE DIFFERENT TRANSFORMATIONS AND TO       +
C                    DEFINE THE INDEX OF THE LAST COLUMN FOR ALL THE           +
C                    TRANSFORMATIONS. ITS VALUE MUST NOT BE LESS THAN 3.       +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY THE  +
C                    SUBROUTINE DURING THE FIRST CALL.                         +
C                    FOR A NEW SELECTION THE USER MUST SET AGAIN INIT TO ZERO. +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           SINIT  - INPUT REAL VALUE. IT MUST CONTAIN THE VALUE OF S   BEFORE +
C                    THE m-TH CALL OF THE SUBROUTINE.                m-1       +
C                                                                              +
C           ITOR   - OUTPUT INTEGER VECTOR. IT REPRESENTS THE ORDER ASSIGNED   +
C                    BY THE SELECTION PROCESS TO EACH TRANSFORMATION DURING    +
C                    THE m-TH CALL. THUS, FOR EXAMPLE, IF ITOR(1)=3,ITOR(2)=7, +
C                    ITOR(3)=1, ... THEN THE FIRST BEST SELECTED TRANSFORM. IS +
C                    T , THE SECOND BEST IS T , THE THIRD ONE IS T  AND SO ON. +
C                     3                      7                    1            +
C                    IT IS DEFINED ONLY AFTER THE 3-TH CALL. BEFORE ITOR(I)=I. +
C                                                                              +
C           REST   - OUTPUT REAL VECTOR CONTAINING THE RESULT GIVEN BY EACH    +
C                    SELECTED TRANSFORMATION AFTER THE m-TH CALL. IT IS        +
C                    DEFINED ONLY AFTER THE THE 3-TH CALL. IN THE FIRST TWO    +
C                    CALLS IT CONTAINS THE VALUES OF S   .                     +
C                                                     m-1                      +
C                    THE RESULTS OF THE SELECTED TRANSFORMATIONS ARE STORED IN +
C                    THE COMPONENTS OF THE VECTOR ACCORDING TO THE ORDER GIVEN +
C                    IN THE DESCRIPTION OF THE SUBROUTINE.                     +
C                                                                              +
C           NDIGIT - INPUT INTEGER VALUE. IT REPRESENTS THE NUMBER OF DIGITS   +
C                    THAT THE USER ALLOWS TO LOSE, DUE TO THE CANCELLATION     +
C                    ERROR, IN THE RHO-ALGORITHM, IN THE EPSILON-ALGORITHM AND +
C                    IN ITS GENERALIZATIONS (SEE THE SUBROUTINE EPSRHA FOR     +
C                    ADDITIONAL INFORMATIONS). WHEN EPSRHA IS CALLED, THE      +
C                    VALUE OF MAXSIN (MAXIMUM NUMBER OF SINGULARITIES) IS SET  +
C                    EQUAL TO THE MAXIMUM COLUMN INDEX MINUS 3, IF THE MAXIMUM +
C                    COLUMN IS GREATER THAN 3. IT IS SET TO ONE OTHERWISE.     +
C                                                                              +
C           IER    - OUTPUT INTEGER VECTOR CONTAINING THE WARNING/ERROR.       +
C                    THIS VECTOR CONTAINS:                                     +
C                    IN IER(I), I= 1, 9 VALUE OF IER FOR THE TRANSFORM. T      +
C                                                                        I     +
C                                       ACCORDING TO THE ORDER GIVEN IN THE    +
C                                       DESCRIPTION (SEE THE MEANING OF IER IN +
C                                       EACH SUBROUTINE).                      +
C                    IN IER(10)         VALUE OF IER FOR THE SUBROUTINE ARRSTO.+
C                    IN IER(11)         VALUE OF IER FOR THE SUBROUTINE SELECT.+
C                                                                              +
C                    FOR IER(11) WE HAVE:                                      +
C                      IER(11) = 200     CALL OF THE SUBROUTINE WITH A NON     +
C                                        ZERO IER VALUE.                       +
C                      IER(11) = 1000    NON ZERO IER VALUE IN SOME SUBROUTINE +
C                                        CALLED BY SELECT.                     +
C                      IER(11) = 1300    THE SIZE OF THE WORKING VECTOR IS     +
C                                        GREATER THAN THE ALLOWED VALUE. THE   +
C                                        SUBROUTINE INCSTO MUST BE USED IN THE +
C                                        CALLING PROGRAM.                      +
C                      IER(11) = 1400    VALUE OF NBC LESS THAN 3.             +
C                                                                              +
C   EXTERNAL MODULES:                                                          +
C                                                                              +
C       -  SUBROUTINES  ARRSTO, EPSRHA, IDELTA, INCSTO, LEVINT, OVERHO,        +
C                       RICHAR, THETA                                          +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE IN +
C          DOUBLE PRECISION AND THE VECTORS ITOR AND REST MUST NOT BE MODIFIED +
C          BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF THE SUBROUTINE.        +
C       -  THE SUBROUTINE WORKS ONLY WITH A VALUE OF NBC NOT GREATER THAN 40.  +
C          THIS IS DUE TO THE PREFIXED MAXIMUM DIMENSION OF THE WORKING ARRAY  +
C          AWRK IN THE COMMON STATEMENT. TO INCREASE THIS VALUE THE USER MUST  +
C          USE, IN THE CALLING PROGRAM, THE SUBROUTINE INCSTO AS EXPLAINED     +
C          IN ITS DESCRIPTION.                                                 +
C       -  TO OBTAIN THE FIRST VALUE OF THE SELECTION THE USER MUST CALL THIS  +
C          SUBROUTINE AT LEAST 3 TIMES.                                        +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE VECTOR      +
C          ARGUMENTS ARE:                                                      +
C           ITOR(9)                                                            +
C           REST(9)                                                            +
C           IER(11)                                                            +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      SUBROUTINE SELECT (NBC, INIT, EPS, SINIT, ITOR, REST,
     *       NDIGIT, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER INIT, NBC, NDIGIT
      DOUBLE PRECISION EPS, SINIT
      INTEGER ITOR(1), IER(1)
      DOUBLE PRECISION REST(1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES AND
C                            CONSTANTS
C
      INTEGER I, IAMAX, INDM, INI, IRM, ITMP, ITRAN, J, MAXSIN, NTR
      INTEGER*4 FIXDIM, IDIM
      DOUBLE PRECISION B, DTMP, SSAV, XINIT, VINIT, THINIT, GINITL,
     *       LINIT, TINIT
      SAVE B, INDM, INI, ITRAN, SSAV, MAXSIN, NTR
C
C                        ... SPECIFICATIONS FOR PARAMETERS
C
      PARAMETER (FIXDIM = 1000)
C
C                        ... SPECIFICATIONS FOR LOCAL ARRAYS
C
      INTEGER NSING(0:3), IA(23), IN(10), INITS(7), IR(9),
     *       JR(9), MAXC(12)
      DOUBLE PRECISION EPSINI(0:3), XINITV(3), DE(9), RESAV(9)
      SAVE INITS, IN, IR, MAXC, RESAV
C
C                        ... COMMON AND EQUIVALENCE STATEMENTS
C
      DOUBLE PRECISION AWRK(FIXDIM)
      INTEGER*4 IWRK(FIXDIM*2)
      EQUIVALENCE (AWRK(1),IWRK(1))
      COMMON /WRKSPR/ AWRK
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         INDM    = -1
         INIT    = 1
         NTR     = 9
         B       = 1.0D0
         ITRAN   = 2
         DO 10 I = 1, NTR
            IER(I)  = 0
            ITOR(I) = I
            IR(I)   = 0
            REST(I) = 0.0D0
            IF (I .LE. 7) INITS(I) = 0
   10    CONTINUE
         IER(10) = 0
         IER(11) = 0
C
         IF (NBC .LT. 3) THEN
            IER(10) = 1400
            IER(11) = 1400
            RETURN
         END IF
C
C                        ... DEFINE THE MAXIMUM WORKING STORAGE
C
         IDIM = FIXDIM
         CALL INCSTO (IDIM)
C
C                        ... COMPUTE THE STORAGE OCCUPATION
C
         CALL ARRSTO (NBC, MAXC, IA, IER(10))
C
C                        ... DEFINE THE INDEXES FOR THE WORKING VECTOR
C
         IN(1)  = IA(4) + 1
         IN(2)  = IA(5) + IN(1)
         IN(3)  = IA(6) + IN(2)
         IN(4)  = IA(23) + IN(3)
         IN(5)  = IA(20) + IN(4)
         IN(6)  = IA(21) + IN(5)
         IN(7)  = IA(14) + IN(6)
         IN(8)  = IA(18) + IN(7)
         IN(9)  = IA(15) + IN(8)
         IN(10) = IA(16) + IN(9)
C
C                        ... CONTROL THE MAXIMUM ARRAY SIZE
C
         INI = IN(10) * 2 - 1
         IAMAX = IN(10) + (IA(7)+1)/2 - 1
         IF (IAMAX .GT. IDIM) THEN
            IER(11) = 1300
            RETURN
         END IF
C
         IF (MAXC(2) .GT. 3) THEN
            MAXSIN = MAXC(2) - 3
         ELSE
            MAXSIN = 1
         END IF
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      DO 20 I = 1, 11
         IF (IER(I) .NE. 0) THEN
            IER(11) = 200
            RETURN
         END IF
   20 CONTINUE
C
C                        ... SAVE THE RESULTS FOR THE FOLLOWING CALL
C
      DO 30 I = 1, NTR
         RESAV(I) = REST(I)
   30 CONTINUE
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1)
C
      INDM = INDM+1
C
C                        ... CALL EPSRHA
C
      DO 40 I = 0, 3
         EPSINI(I) = SINIT
   40 CONTINUE
      XINITV(1) = 1.0D0 + 1.0D0/DBLE(INDM+1)
      XINITV(2) = XINITV(1)
      XINITV(3) = DBLE(INDM)
      CALL EPSRHA (AWRK(1), MAXC(2), INITS(1), EPS, EPSINI, NDIGIT,
     *  MAXSIN, NSING, IWRK(INI), AWRK(IN(1)), AWRK(IN(2)), XINITV,
     *  IER)
      IER(5) = IER(2)
      IER(6) = IER(3)
      IER(2) = IER(4)
      IF (IER(1) .NE. 0 .OR. IER(2) .NE. 0 .OR. IER(5) .NE. 0 .OR.
     *   IER(6) .NE. 0 ) THEN
         IER(11) = 1000
         RETURN
      END IF
      REST(1) = EPSINI(0)
      REST(5) = EPSINI(1)
      REST(6) = EPSINI(2)
      REST(2) = EPSINI(3)
C
C                        ... CALL THETA
C
      THINIT = SINIT
      CALL THETA (AWRK(IN(3)), MAXC(12), INITS(2), EPS, THINIT, IER(3))
      IF (IER(3) .NE. 0) THEN
         IER(11) = 1000
         RETURN
      END IF
      REST(3) = THINIT
      IF (INDM .NE. 0) THEN
C
C                        ... CALL RICHAR
C
         TINIT = SSAV
         XINIT = SINIT - SSAV
         CALL RICHAR (AWRK(IN(4)), MAXC(10), INITS(3), EPS, TINIT,
     *      AWRK(IN(5)), XINIT, IER(4))
         IF (IER(4) .NE. 0) THEN
            IER(11) = 1000
            RETURN
         END IF
         REST(4) = TINIT
         IF (INDM .EQ. 1) REST(4) = SINIT
      ELSE
         REST(4) = SINIT
      END IF
      SSAV   = SINIT
C
C                        ... CALL IDELTA
C
      XINIT = SINIT
      CALL IDELTA (AWRK(IN(6)), MAXC(5), INITS(4), EPS, XINIT, IER(7))
      IF (IER(7) .NE. 0) THEN
         IER(11) = 1000
         RETURN
      END IF
      REST(7) = XINIT
C
C                        ... CALL OVERHO
C
      VINIT = SINIT
      CALL OVERHO (AWRK(IN(7)), MAXC(8), INITS(5), EPS, VINIT, IER(8))
      IF (IER(8) .NE. 0) THEN
         IER(11) = 1000
         RETURN
      END IF
      REST(8) = VINIT
      IF (INDM .EQ. 1) REST(8) = SINIT
C
C                        ... CALL LEVINT
C
      LINIT  = SINIT
      CALL LEVINT (AWRK(IN(8)), AWRK(IN(9)), MAXC(6), INITS(6), EPS,
     *     LINIT, GINITL, B, ITRAN, IER(9))
      IF (IER(9) .NE. 0) THEN
         IER(11) = 1000
         RETURN
      END IF
      REST(9) = LINIT
      IF (INDM .EQ. 1) REST(9) = SINIT
C
C                        ... CONTROL THE BEGINNING OF THE SELECTION
C
      IF (INDM .LT. 2) RETURN
C
C                        ... SELECTION BETWEEN THE TRANSFORMATIONS
C
      DO 50 I = 1, NTR
         ITOR(I) = I
         DE(I) = DABS(REST(I)-RESAV(I))
   50 CONTINUE
C
      DO 70 I = 1, NTR-1
         ITMP  = ITOR(I)
         DTMP  = DE(I)
         DO 60 J = I+1, NTR
            IF (DE(J) .LT. DTMP) THEN
               DE(I)   = DE(J)
               DE(J)   = DTMP
               DTMP    = DE(I)
               ITOR(I) = ITOR(J)
               ITOR(J) = ITMP
               ITMP    = ITOR(I)
            END IF
   60    CONTINUE
   70 CONTINUE
C
      DO 80 I = 1, NTR
         J       = ITOR(I)
         IR(J)   = IR(J) + NTR + 1 - I
         JR(J)   = IR(J)
         ITOR(I) = I
   80 CONTINUE
C
      DO 100 I = 1, NTR-1
         ITMP = ITOR(I)
         IRM  = JR(I)
         DO 90 J = I+1, NTR
            IF (JR(J) .GT. IRM) THEN
               JR(I)   = JR(J)
               JR(J)   = IRM
               IRM     = JR(I)
               ITOR(I) = ITOR(J)
               ITOR(J) = ITMP
               ITMP    = ITOR(I)
            END IF
   90    CONTINUE
  100 CONTINUE
      RETURN
      END
