C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   SUBROUTINE NAME     - VGTRAN                                               +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                              +
C   DESCRIPTION:                                                               +
C                                                                              +
C           VECTOR SEQUENCE EXTRAPOLATION.                                     +
C           THE SUBROUTINE PERFORMS THE VECTOR G-TRANSFORMATION.               +
C                                                                              +
C   USAGE:                                                                     +
C           CALL VGTRAN (GMAT, MAXCOL, INIT, EPS, GINIT, RSINIT,               +
C              RSVEC, GAUX, IR, IP, IER)                                       +
C                                                                              +
C   ARGUMENTS:                                                                 +
C                                                                              +
C           GMAT   - OUTPUT REAL MATRIX. IN THE CALLING PROCEDURE THE          +
C                    DIMENSION OF GMAT MUST BE (IP,0:IE), WITH IE GREATER      +
C                    OR EQUAL TO (2*MAXCOL-1). THIS MATRIX CONTAINS, AFTER     +
C                    THE k-TH CALL:                                            +
C                                                                              +
C                     - IF k = 1                                               +
C                                                                              +
C                        (0)                                                   +
C                       G                                                      +
C                        0                                                     +
C                                                                              +
C                     - IF 1 < k <= 2*MAXCOL+1 AND k ODD                       +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                 (3)         (0)        (1)                   +
C                                G       ,   G       ,  G                      +
C                                 (k-1)/2-1   (k-1)/2    (k-1)/2               +
C                                                                              +
C                     - IF 1 < k <  2*MAXCOL+1 AND k EVEN                      +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                 (1)         (2)        (0)                   +
C                                G       ,   G       ,  G                      +
C                                 k/2-1       k/2-1      k/2                   +
C                                                                              +
C                     - IF k > 2*MAXCOL+1                                      +
C                                                                              +
C                        (k-1)   (k-3)   (k-2)   (k-5)   (k-4)                 +
C                       G    ,  G    ,  G    ,  G   ,   G     ,...,            +
C                        0       1       1       2       2                     +
C                                  (k-2*MAXCOL-1)    (k-2*MAXCOL)              +
C                                 G             ,   G                          +
C                                  MAXCOL            MAXCOL                    +
C                                                                              +
C           MAXCOL - INPUT INTEGER GIVING THE INDEX OF THE LAST COLUMN THAT    +
C                                                                   (j)        +
C                    THE USER WANTS TO COMPUTE (THE LOWER INDEX OF G   ).      +
C                                                                   m          +
C                                                                              +
C           INIT   - INPUT/OUTPUT INTEGER TO BE SET TO ZERO BEFORE THE FIRST   +
C                    CALL OF THE SUBROUTINE. ITS VALUE IS CHANGED TO 1 BY      +
C                    THE SUBROUTINE DURING THE FIRST CALL.                     +
C                    FOR A NEW APPLICATION OF THE ALGORITHM THE USER           +
C                    MUST SET AGAIN INIT TO ZERO.                              +
C                                                                              +
C           EPS    - INPUT REAL VALUE USED IN TESTS FOR ZERO.                  +
C                    IF DABS(X) .LT. EPS, THEN X IS CONSIDERED TO BE ZERO.     +
C                                                                              +
C           GINIT  - INPUT/OUTPUT REAL VECTOR. IT MUST CONTAIN THE VALUE       +
C                    OF S   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.         +
C                        k-1                                                   +
C                    GINIT CONTAINS AFTER THE k-TH CALL OF THE SUBROUTINE      +
C                                                                              +
C                     (0)                                                      +
C                    G            IF  k = 1                                    +
C                     0                                                        +
C                                                                              +
C                     (0)                                                      +
C                    G            IF  1 < k <  2*MAXCOL+1, k EVEN              +
C                     k/2                                                      +
C                                                                              +
C                     (1)                                                      +
C                    G            IF  1 < k <= 2*MAXCOL+1, k ODD               +
C                     (k-1)/2                                                  +
C                                                                              +
C                     (k-2*MAXCOL)                                             +
C                    G            IF  k > 2*MAXCOL+1                           +
C                     MAXCOL                                                   +
C                                                                              +
C                    THE SUBROUTINE RETURNS THE SEQUENCE                       +
C                                                                              +
C                     (0)  (0)  (1)  (0)  (1)      (0)     (1)     (2)         +
C                    G ,  G  , G  , G  , G  ,..., G     , G     , G     , ...  +
C                     0    1    1    2    2        MAXCOL  MAXCOL  MAXCOL      +
C                                                                              +
C           RSINIT - INPUT REAL VALUE. IT MUST CONTAIN THE ELEMENT             +
C                    x   , BEFORE THE  k-TH CALL OF THE SUBROUTINE.            +
C                     k-1                                                      +
C                                                                              +
C           RSVEC  - WORKING REAL VECTOR. IN THE CALLING PROCEDURE THE         +
C                    DIMENSION OF RSVEC MUST BE (0:IE), WITH IE GREATER        +
C                    OR EQUAL TO 2*(MAXCOL-1). THIS VECTOR CONTAINS THE        +
C                    r AND s VALUES.                                           +
C                                                                              +
C           GAUX   - WORKING REAL VECTOR. IN THE CALLING PROCEDURE THE         +
C                    DIMENSION OF GAUX MUST BE IP.                             +
C                                                                              +
C           IR     - INPUT ROWS DIMENSION EXACTLY AS SPECIFIED IN THE          +
C                    DIMENSION STATEMENTS IN THE CALLING PROGRAM FOR THE       +
C                    MATRIX GMAT.                                              +
C                                                                              +
C           IP     - INPUT DIMENSION OF THE VECTOR SPACE (NUMBER OF ROWS OF    +
C                    THE MATRIX GMAT AND COMPONENTS OF THE VECTORS GINIT AND   +
C                    GAUX).                                                    +
C                                                                              +
C           IER    - INPUT/OUTPUT INDEX WARNING/ERROR.                         +
C                    IER = 100     DIVISION BY ZERO IN THE SUBROUTINE.         +
C                    IER = 200     CALL OF THE SUBROUTINE WITH A NON ZERO      +
C                                  IER VALUE.                                  +
C                    IER = 300     ROWS DIMENSION TOO SMALL IN THE CALLING     +
C                                  PROCEDURE COMPARED TO THE VECTOR SPACE      +
C                                  DIMENSION.                                  +
C                    IER = 400     WARNING. CANCELLATION ERROR. NUMERICAL      +
C                                  INSTABILITY PRESENT. THE USER MUST SET      +
C                                  AGAIN IER=0 IN THE CALLING PROGRAM TO       +
C                                  CONTINUE.                                   +
C                                                                              +
C   REMARKS:                                                                   +
C                                                                              +
C       -  ALL THE REAL ARGUMENTS PASSED FROM THE CALLING PROCEDURE MUST BE    +
C          IN DOUBLE PRECISION. THE MATRIX GMAT AND THE VECTORS RSVEC AND GAUX +
C          MUST NOT BE MODIFIED BY THE USER BETWEEN TWO CONSECUTIVE CALLS OF   +
C          THE SUBROUTINE.                                                     +
C                                  (0)                                         +
C       -  TO OBTAIN THE VALUE OF G   (J <= MAXCOL) THE USER MUST CALL THIS    +
C                                  J                                           +
C          SUBROUTINE 2*J TIMES.                                               +
C       -  IN THE CALLING PROCEDURE THE MINIMAL DIMENSIONS FOR THE MATRIX AND  +
C          VECTORS ARGUMENTS (IR=IP) ARE:                                      +
C           GMAT(IP,0:2*MAXCOL-1)                                              +
C           GINIT(IP)                                                          +
C           GAUX(IP)                                                           +
C           RSVEC(0:2*(MAXCOL-1))                                              +
C                                                                              +
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                
      SUBROUTINE VGTRAN (GMAT, MAXCOL, INIT, EPS, GINIT, RSINIT,      
     *       RSVEC, GAUX, IR, IP, IER)
C
C                        ... SPECIFICATIONS FOR ARGUMENTS
C
      INTEGER IER, INIT, IR, IP, MAXCOL
      DOUBLE PRECISION EPS, RSINIT
      DOUBLE PRECISION GAUX(1), GINIT(1), GMAT(IR,0:1), RSVEC(0:1)
C
C                        ... SPECIFICATIONS FOR LOCAL VARIABLES   
C
      INTEGER II, IND, INDM, INDS, INDSM1, MAXC2
      DOUBLE PRECISION AUX0, AUX1, AUX2, TMP
      SAVE INDM, MAXC2
C
C                        ... EXECUTABLE STATEMENTS
C
      IF (INIT .EQ. 0) THEN
C
C                        ... FIRST CALL OF THE SUBROUTINE
C
         IF (IP .GT. IR) THEN
            IER = 300
            RETURN
         END IF
C
         IER      = 0                                                           
         INIT     = 1
         INDM     = 0
         RSVEC(0) = RSINIT
         DO 10 II = 1, IP
            GMAT(II,0)  = GINIT(II)
   10    CONTINUE
         MAXC2    = 2*MAXCOL - 1
         RETURN
      END IF
C
C                        ... NEXT CALLS OF THE SUBROUTINE
C
      IF (IER .NE. 0) THEN
         IER = 200
         RETURN
      END IF
C
C
      IF (MAXCOL .EQ. 0) THEN
         RSVEC(0) = RSINIT
         DO 20 II = 1, IP
            GMAT(II,0)  = GINIT(II)
   20    CONTINUE
         RETURN
      END IF
C
C                        ... INDM REPRESENTS THE (NUMBER_OF_CALLS - 1).
C
      INDM = INDM + 1 
C
C                        ... IF INDM > 2*MAXCOL-1 THEN 
C                                         (NUMBER_OF_CALLS-2*MAXCOL)
C                            THE ELEMENT G 
C                                         MAXCOL
C                            IS COMPUTED.
C
      IF (INDM .LE. MAXC2) THEN
         IND = INDM
      ELSE 
         IND = MAXC2 
      END IF
C
C                        ... INITIALIZATION FOR THE AUXILIARY VARIABLE 
C                            AUX1.
C
      AUX1   = RSINIT
C  
C                        ... STARTING LOOP FOR COMPUTING THE ELEMENTS.
C
      DO 40 INDS = 0, IND - 1
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENTS r OR s
C
         IF (DABS(RSVEC(INDS)) .LT. EPS) THEN
            IER = 100
            RETURN
         END IF
         TMP  = AUX1 / RSVEC(INDS) - 1.0D0
         IF (DABS(TMP) .LT. EPS) IER = 400
         AUX0 = TMP
         IF (INDS .NE. 0) AUX0 = RSVEC(INDS-1) * AUX0
C  
C                        ... COMPUTE THE VALUE OF THE ELEMENTS G
C
         IF (MOD(INDS,2) .EQ. 0) THEN
            IF (DABS(TMP) .LT. EPS) THEN
               IER = 100
               RETURN
            END IF
            INDSM1 = INDS - 1
            DO 30 II = 1, IP
               IF (INDS .EQ. 0) THEN
                  GAUX(II)   = GMAT(II,0)-(GINIT(II)-GMAT(II,0))/TMP
                  GMAT(II,0) = GINIT(II)
                  IF (INDM .LE. 2) GINIT(II) = GAUX(II)
               ELSE
                  GINIT(II)  = GMAT(II,INDSM1) - 
     *              (GMAT(II,INDS) - GMAT(II,INDSM1)) / TMP
C  
C                        ... SHIFT THE ELEMENTS OF THE G ARRAY.
C
                  GMAT(II,INDSM1)  = GMAT(II,INDS)
                  GMAT(II,INDS)    = GAUX(II)
                  GAUX(II)         = GINIT(II)
               END IF
   30       CONTINUE
         END IF
C  
C                        ... SHIFT THE ELEMENTS OF THE rs VECTOR.
C
         IF (INDS .NE. 0) RSVEC(INDS-1) = AUX2
         AUX2 = AUX1
         AUX1 = AUX0
C  
C                        ... END LOOP FOR INDS.
C
   40 CONTINUE
C  
C                        ... SAVE FOR THE FINAL VALUES.
C
      RSVEC(IND-1) = AUX2
      IF (IND .LT. MAXC2) RSVEC(IND) = AUX1
      DO 50 II = 1, IP
         GMAT(II,IND) = GINIT(II)
   50 CONTINUE
      RETURN
      END
